
/* KeyConfig */

/* 
By Ingemar Ragnemalm 1995
This program shows a modal dialog box, in which you can select eight keys,
supposedly fr keyboard control for a game, plus a three-button set of radio
buttons and a checkbox. It also shows the key codes found for each key.

You should study this program when making your own key configuration dialogs.
The most important message is that you should get the key codes through events,
not assume that everybody has the same key configuration as you have.
*/

/* This version of KeyConfig uses the DialogUtils unit in order to
simplify the handling of dialog items. */

#include "MiniPreferences.h"
#include "DialogUtils.h"



/* Prefs record */

typedef struct
{
	char	keyCodes[8];	/* Key codes */
	char	keyChars[8];	/* ASCII values */
	short		myRadio;
	Boolean	myCheckBox;
} PrefsRec;

typedef PrefsRec **PrefsHnd;


PrefsHnd gPrefs;

static char	gTempKeyCodes[8];	/* Key codes, temporary stoareg while the dialog is open */

/* The following constants must match the DLOG/DITL resource! */
#define kHighDlogRes 128
#define	kOKButton		1
#define	kCancelButton	2
#define	kFirstKey		3
#define	kLastKey		10
#define	kFirstRadio		11
#define	kLastRadio		13
#define	kCheckBox		14
#define	kFirstCode		15
#define	kLastCode		22


/* StdFilter */

/* StdFilter is a dialog filter function that is useful for most
modal dialogs, as long as they have an OK and a Cancel button.
It maps return and enter to OK, and command-period and ESC to Cancel.
It also draws a frame around the OK button. */

static pascal Boolean StdFilter(DialogPtr theDialog, EventRecord *theEvent, short *itemHit)
{
	char theChar;
	short kind;
	Handle item;
	Rect box;

	switch ( theEvent->what )
		{
		case keyDown: 
				theChar = (char)(theEvent->message & charCodeMask);
				if ( (((theEvent->modifiers & cmdKey) != 0) && (theChar == '.')) || (theChar == (char)27) ) /*cmd-. or ESC*/
					{
						*itemHit = kCancelButton;
/*Highlight the cancel button*/
						GetDialogItem(theDialog, kCancelButton, &kind, &item, &box);
						HiliteControl((ControlHandle)item, 1);

						return true;
					};
				if ( (theChar == (char)13) || (theChar == (char)3) )
					{
						*itemHit = kOKButton;
/*Highlight the OK button*/
						GetDialogItem(theDialog, 1, &kind, &item, &box);
						HiliteControl((ControlHandle)item, kOKButton);

						return true;
					};
				break; /*keyDown*/
		case updateEvt:
				BeginUpdate(theDialog);
				SetPort(theDialog);

				DrawDialog(theDialog);

/*Frame default button - item 1*/
				GetDialogItem(theDialog, kOKButton, &kind, &item, &box);
				InsetRect(&box, -4, -4);
				PenSize(3, 3);
				FrameRoundRect(&box, 15, 15);

				EndUpdate(theDialog);
				break; /*update event*/
	}; /*case*/
	return false;
} /*StdFilter*/



static pascal Boolean KeysFilter(DialogPtr theDialog, EventRecord *theEvent, short *itemHit)
{
	short	item;
	short	theKeyNumber;
	Str255	str;

	if ( (theEvent->modifiers & cmdKey) != 0 )			/*Command key held down*/
		return StdFilter(theDialog, theEvent, itemHit);
	else if ( theEvent->what != keyDown )				/*Not a keydown*/
		return StdFilter(theDialog, theEvent, itemHit);
	else if ( (theEvent->message & charCodeMask) == 3 ) /*Enter*/
		return StdFilter(theDialog, theEvent, itemHit);
	else if ( (theEvent->message & charCodeMask) == 13 ) /*Return*/
		return StdFilter(theDialog, theEvent, itemHit);
	else if ( (theEvent->message & charCodeMask) == 27 ) /*Escape*/
		return StdFilter(theDialog, theEvent, itemHit);
	else if ( (theEvent->message & charCodeMask) == 9 ) /*TAB*/
		return StdFilter(theDialog, theEvent, itemHit);
	else
	{
/* Get the key code */
		theKeyNumber = (theEvent->message & keyCodeMask) >> 8;
		
		if ( IsDialogEvent(theEvent) )
			if ( DialogSelect(theEvent, &theDialog, &item) )
			{
				*itemHit = item;

/* If it is one of the key boxes, put the key code in the temprary array */
				if (*itemHit >= kFirstKey && *itemHit <= kLastKey)
				{
					gTempKeyCodes[*itemHit - kFirstKey] = theKeyNumber;

/* Just for the demo, I also display the key codes. A real game would never do that. */
					NumToString(theKeyNumber, str);
					SetTextDItem(theDialog, *itemHit - kFirstKey + kFirstCode, str);

					return true;
				}
			}
	}
}; /* KeysFilter*/


/* AskKeys */
/* This is the procedure that handles the dialog. */

static Boolean AskKeys(PrefsHnd prefs)
{
	DialogPtr theDialog;
	GrafPtr oldPort;
	short itemHit;
	Str255 str;
	short	itemNo;
	Boolean	returnValue = false;

	GetPort(&oldPort);
	theDialog = GetNewDialog(kHighDlogRes, nil, (WindowPtr)-1);
	ShowWindow(theDialog);
	SelectWindow(theDialog);
	SetPort(theDialog);

/* Set all dialog items to their initial values */

/* First set all the editable text boxes */
	for (itemNo = kFirstKey; itemNo <= kLastKey; itemNo++)
	{
		str[0] = 1;											/* Length 1 */
		str[1] = (**prefs).keyChars[itemNo - kFirstKey];	/* Put the char in the string */
		SetTextDItem(theDialog, itemNo, str);
	}

/* then the radio buttons */
	for (itemNo = kFirstRadio; itemNo <= kLastRadio; itemNo++)
	{
		SetBooleanDItem(theDialog, itemNo, itemNo == (**prefs).myRadio);
	}

/* and the checkbox */	
	SetBooleanDItem(theDialog, itemNo, (**prefs).myCheckBox);

/* Finally, copy all the key codes to a temporary storage. */
	for (itemNo = 0; itemNo <= kLastKey - kFirstKey; itemNo++)
	{
		gTempKeyCodes[itemNo] = (**prefs).keyCodes[itemNo];

/* Just for the demo, I also display the key codes. A real game would never do that. */
		NumToString((**prefs).keyCodes[itemNo], str);
		SetTextDItem(theDialog, itemNo + kFirstCode, str);
	}


/* One more thing: let the first key be selected */
	SelectDialogItemText(theDialog, kFirstKey, 0, 1);

	itemHit = -1;
	while ( (itemHit != 1) && (itemHit != 2) ) /* 1=ok, 2=cancel */
	{
		ModalDialog(&KeysFilter, &itemHit);

/* Handle radio buttons and checkboxes here */
		if (itemHit == kCheckBox)
		{
			ToggleBooleanDItem(theDialog, itemHit);
		}

		if (itemHit >= kFirstRadio && itemHit <= kLastRadio)
			for (itemNo = kFirstRadio; itemNo <= kLastRadio; itemNo++)
			{
				SetBooleanDItem(theDialog, itemNo, itemNo == itemHit);
			}

/* If it was a key, select it */
		if (itemHit >= kFirstKey && itemHit <= kLastKey)
			SelectDialogItemText(theDialog, itemHit, 0, 32767);

	} /*while*/

/* If not cancelled, put the results in the prefs structure. */

	if ( itemHit == 1 )
	{
/* Get values from the dialog items */

/* First get all the editable text boxes */
		for (itemNo = kFirstKey; itemNo <= kLastKey; itemNo++)
		{
			GetTextDItem(theDialog, itemNo, str);
			if (str[0] != 0)			/* If it's empty, leave it alone. */
				(**prefs).keyChars[itemNo - kFirstKey] = str[1];
		}

/* then the radio buttons */
		for (itemNo = kFirstRadio; itemNo <= kLastRadio; itemNo++)
		{
			if (GetBooleanDItem(theDialog, itemNo))
				(**prefs).myRadio = itemNo;
		}

/* and the checkbox */	
		(**prefs).myCheckBox = GetBooleanDItem(theDialog, kCheckBox);

/* Finally, copy all the key codes from the temporary storage. */
		for (itemNo = 0; itemNo <= kLastKey - kFirstKey; itemNo++)
			(**prefs).keyCodes[itemNo] = gTempKeyCodes[itemNo];

		returnValue = true;	/* OK */
	};
	DisposeDialog(theDialog);
	SetPort(oldPort);
	return returnValue;
} /*AskKeys*/



/*InitPrefs*/
/*This procedure loads the preferences from a resource in the current resource file,*/
/*creating a new one if there is none.*/

static void InitPrefs()
{
	gPrefs = (PrefsHnd)Get1Resource('Pref', 0);
	if ( gPrefs == nil ) /*Didn't exist - create it!*/
		{
		gPrefs = (PrefsHnd)NewHandle(sizeof(PrefsRec));
		if ( gPrefs == nil )
			ExitToShell();							/*Insert error message here*/

/* We should fill it with some values here */

		AddResource((Handle)gPrefs, 'Pref', 0, "\pPreferences");
		}
	else /*Did exist - check the size!*/
		if ( GetHandleSize((Handle)gPrefs) < sizeof(PrefsRec) )
			SetHandleSize((Handle)gPrefs, sizeof(PrefsRec));
}; /*InitScores*/


/* Standard inits */

static void InitToolbox(void) {
	InitGraf (&qd.thePort);
	InitFonts ();
	FlushEvents (everyEvent,0);
	InitWindows ();
	InitMenus ();
	TEInit ();
	InitDialogs (nil);
	InitCursor ();
}

/****************** Main program ******************/

void main(void)
{
	short gAppFile, gPrefFile;

	InitToolbox();

/*Get the prefs file - create as necessary.*/
	if ( SetPrefFile("\pKeyConfig Preferences", '????', 'pref', &gAppFile, &gPrefFile) )
		;

/*Set the current resource file to the preference file before InitScores*/
	if ( gPrefFile != 0 )
		UseResFile(gPrefFile);
	InitPrefs();
	UseResFile(gAppFile);

	if (AskKeys(gPrefs))
	{
		ChangedResource((Handle)gPrefs);
		WriteResource((Handle)gPrefs);
	}
	
	CloseResFile(gPrefFile);

} /*KeyConfig*/
