//
//	File: sprite.c
//
//	This file contains the routines to draw the sprites
//
//	2/19/95 -- Created by Mick
//

// include files

#include "global.h"

#include "sprite.h"

#include "main.h"

// defines for this file

#define kEndShapeToken					0L				// the end of shape maker
#define kLineStartToken					1L				// the line start marker
#define kDrawPixelsToken			2L				// the draw run marker
#define kSkipPixelsToken				3L				// the skip pixels marker

// global function declarations

tSpriteInfo *loadSprite( signed short inSpriteResID );
void disposeSprite( tSpriteInfo *inSpriteInfo );
void startSpriteDraw( Rect *inClipRect, PixMapHandle inDestPixMap );
void endSpriteDraw( void );
void drawSprite( tSpriteInfo *inSpriteInfo, Point inWhere );

// global data owned by this file

// local function declarations

static void renderSpriteClipped( tSpriteInfo *inSpriteInfo, Rect *inDestRect );
static void renderSpriteUnclipped( tSpriteInfo *inSpriteInfo, Rect *inDestRect );

static void drawSprite201( Rect *inDestRect );
static void drawSprite202( Rect *inDestRect );
static void drawSprite203( Rect *inDestRect );
static void drawSprite204( Rect *inDestRect );
static void drawSprite205( Rect *inDestRect );
static void drawSprite206( Rect *inDestRect );
static void drawSprite207( Rect *inDestRect );
static void drawSprite208( Rect *inDestRect );
static void drawSprite209( Rect *inDestRect );
static void drawSprite210( Rect *inDestRect );

// static data

Rect sClipRect;																// the rectangle to clip to
PixMapHandle sDestPixMap;						// where we are going to draw this data
unsigned char *sBaseAddr;					// the base address of the pixmap
unsigned long sRowBytes;						// the row bytes of the pixmap

// functions

//
//	loadSprite -
//
//	Loads/allocates all the data for a sprite.
//

tSpriteInfo *loadSprite( signed short inSpriteResID )
{
	tSpriteInfo *newSprite;			// the new sprite data
	
	// create the sprite info record
	newSprite = ( tSpriteInfo * )NewPtr( sizeof( tSpriteInfo ) );
	
	// load the sprite
	newSprite->fSpriteData = GetResource( kSpriteResType, inSpriteResID );
	HNoPurge( newSprite->fSpriteData );
	if ( newSprite->fSpriteData == ( Handle )kNil )
		{
			// if it did not load, drop into the debugger -- real programs would have error checking
			Debugger();
		}
	
	// copy its bounds rect
	newSprite->fSpriteRect = **( ( Rect ** )( newSprite->fSpriteData ) );
	
	// install the function pointer
	switch( inSpriteResID )
		{
			case 201:
				newSprite->fSpriteFunction = &drawSprite201;
				break;
			case 202:
				newSprite->fSpriteFunction = &drawSprite202;
				break;
			case 203:
				newSprite->fSpriteFunction = &drawSprite203;
				break;
			case 204:
				newSprite->fSpriteFunction = &drawSprite204;
				break;
			case 205:
				newSprite->fSpriteFunction = &drawSprite205;
				break;
			case 206:
				newSprite->fSpriteFunction = &drawSprite206;
				break;
			case 207:
				newSprite->fSpriteFunction = &drawSprite207;
				break;
			case 208:
				newSprite->fSpriteFunction = &drawSprite208;
				break;
			case 209:
				newSprite->fSpriteFunction = &drawSprite209;
				break;
			case 210:
				newSprite->fSpriteFunction = &drawSprite210;
				break;
			default:
				newSprite->fSpriteFunction = ( tSpriteFunctionPtr )kNil;
				break;
		}
	
	// return the new sprite!
	return newSprite;
}


//
//	disposeSprite -
//
//	Disposes/releases all the memory used in a sprite
//

void disposeSprite( tSpriteInfo *inSpriteInfo )
{
	// dump the sprite resource
	ReleaseResource( inSpriteInfo->fSpriteData );
	
	// free the structure
	DisposePtr( ( Ptr )inSpriteInfo );
}


//
//	startSpriteDraw -
//
//	Prepare the sprite draw. Assumes that the port is set to the destination port.
//

void startSpriteDraw( Rect *inClipRect, PixMapHandle inDestPixMap )
{
	// set the clip region to be the passed in rect
	sClipRect = *inClipRect;
	
	// save the pix map info (so we can use it)
	sDestPixMap = inDestPixMap;

	// get info from the pix map
	sBaseAddr = ( unsigned char * )GetPixBaseAddr( sDestPixMap );
	sRowBytes = ( *sDestPixMap )->rowBytes & 0x3fff;
}


//
//	endSpriteDraw -
//
//	End the sprite draw sequence.
//

void endSpriteDraw( void )
{
}


//
//	drawSprite -
//
//	Draw the sprite in the port.
//

void drawSprite( tSpriteInfo *inSpriteInfo, Point inWhere )
{
	Rect destRect;		// where we want to draw the sprite
	
	// calculate the destination rect
	destRect = inSpriteInfo->fSpriteRect;
	OffsetRect( &destRect, inWhere.h, inWhere.v );
	
	// determine if the spite needs to be drawn at all
	if( destRect.top >= sClipRect.bottom || destRect.bottom <= sClipRect.top ||
			destRect.left >= sClipRect.right || destRect.right <= sClipRect.left )
		{
			// no need to draw, goodbye
			return;
		}
	
	// determine if the sprite will be clipped
	if ( destRect.top < sClipRect.top || destRect.bottom > sClipRect.bottom ||
			destRect.left < sClipRect.left || destRect.right > sClipRect.right )
		{
			// handle the clipped case
			renderSpriteClipped( inSpriteInfo, &destRect );
		}
	else
		{
			// handle the unclipped case
			( *( inSpriteInfo->fSpriteFunction ) )( &destRect );
		}
}


//
//	renderSpriteClipped -
//
//	Draw the spite, deal with clipping.
//

void renderSpriteClipped( tSpriteInfo *inSpriteInfo, Rect *inDestRect )
{
	Rect clipRect;																	// the rect that defines the clipped shape
	unsigned char *rowStart;						// the pointer to the start of this row
	unsigned char *srcPtr;								// the current position in the sprite data
	unsigned char *destPtr;							// the current position in the destination pixmap
	unsigned long miscCounter;					// a counter for various purposes
	unsigned long extraCounter;				// a counter for right clippling purposes ( how much extra was there? )
	unsigned long tokenOp;									// the op code from the token
	unsigned long tokenData;							// the data from the token
	unsigned char exitFlag;								// should we exit from the loop?
	unsigned long yCount;									// how many lines down in the shape are we?
	unsigned long xCount;									// where are we in this line?
	
	// create a clipped rect in the coordinates of the sprite
	clipRect.left = inDestRect->left < sClipRect.left ? sClipRect.left - inDestRect->left : 0;
	clipRect.right = inDestRect->right > sClipRect.right ? sClipRect.right - inDestRect->left : inDestRect->right - inDestRect->left;
	clipRect.top = inDestRect->top < sClipRect.top ? sClipRect.top - inDestRect->top : 0;
	clipRect.bottom = inDestRect->bottom > sClipRect.bottom ? sClipRect.bottom - inDestRect->top : inDestRect->bottom - inDestRect->top;

	// set up the counters
	yCount = 0;

	// determine characteristics about the pixmap
	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;
	
	// move to the right place in the shape ( just past the size rect )
	srcPtr = ( unsigned char * )( ( *( inSpriteInfo->fSpriteData ) ) + sizeof( Rect ) );
	
	// loop until we are done
	exitFlag = kFalse;
	while( !exitFlag )
		{
			// get a token
			tokenOp = ( *( ( unsigned long * )srcPtr ) ) >> 24;
			tokenData = ( *( ( unsigned long * )srcPtr ) ) & 0x00ffffff;
			srcPtr += sizeof( unsigned long );
			
			// depending on the token
			switch( tokenOp )
				{
					case kDrawPixelsToken:
						miscCounter = tokenData;
						extraCounter = 0;
						
						// if we need to, clip to the left
						if( xCount < clipRect.left )
							{
								// if this run does not appear at all, don't draw it
								if ( miscCounter < clipRect.left - xCount )
									{
										destPtr += miscCounter;
										srcPtr += miscCounter;
										srcPtr += ( ( tokenData & 3L ) == 0 ) ? 0 : ( 4 - ( tokenData & 3L ) );
										xCount += miscCounter;
										break;
									}
								else
									{
										// if it does, skip to where we can draw
										miscCounter -= clipRect.left - xCount;
										destPtr += clipRect.left - xCount;
										srcPtr += clipRect.left - xCount;
										xCount += clipRect.left - xCount;
									}
							}
						
						// if we need to, clip to the right
						if ( xCount + miscCounter > clipRect.right )
							{
								// if this run does not appear at all, skip it
								if ( xCount > clipRect.right )
									{
										destPtr += miscCounter;
										srcPtr += miscCounter;
										srcPtr += ( ( tokenData & 3L ) == 0 ) ? 0 : ( 4 - ( tokenData & 3L ) );
										xCount += miscCounter;
										break;
									}
								else
									{
										// if it does, setup to draw what we can
										extraCounter = miscCounter;
										miscCounter -= ( xCount + miscCounter ) - clipRect.right;
										extraCounter -= miscCounter;
									}
							}
						
						// adjust xCount for the run
						xCount += miscCounter;
						
						// move data in the biggest chunks we can find
 #ifdef powerc
 						// move in doubles while we can
						while( miscCounter >= sizeof( double ) )
							{
								*( ( double * )destPtr ) = *( ( double * )srcPtr );
								destPtr += sizeof( double );
								srcPtr += sizeof( double );
								miscCounter -= sizeof( double );
							}
						
						// move a long if we can
						if ( miscCounter >= sizeof( unsigned long ) )
							{
								*( ( unsigned long * )destPtr ) = *( ( unsigned long  * )srcPtr );
								destPtr += sizeof( unsigned long  );
								srcPtr += sizeof( unsigned long  );
								miscCounter -= sizeof( unsigned long  );
							}
#else
						// move in longs while we can
						while( miscCounter >= sizeof( unsigned long ) )
							{
								*( ( unsigned long * )destPtr ) = *( ( unsigned long * )srcPtr );
								destPtr += sizeof( unsigned long );
								srcPtr += sizeof( unsigned long );
								miscCounter -= sizeof( unsigned long );
							}
#endif						
												
						// move a short if we can
						if ( miscCounter >= sizeof( unsigned short ) )
							{
								*( ( unsigned short * )destPtr ) = *( ( unsigned short  * )srcPtr );
								destPtr += sizeof( unsigned short  );
								srcPtr += sizeof( unsigned short  );
								miscCounter -= sizeof( unsigned short  );
							}

						// move a char if we can
						if ( miscCounter >= sizeof( unsigned char ) )
							{
								*( ( unsigned char  * )destPtr ) = *( ( unsigned char  * )srcPtr );
								destPtr += sizeof( unsigned char  );
								srcPtr += sizeof( unsigned char  );
								miscCounter -= sizeof( unsigned char );
							}
						
						// adjust for right clipping
						destPtr += extraCounter;
						srcPtr += extraCounter;
						xCount += extraCounter;
						
						// adjust for the padding
						srcPtr += ( ( tokenData & 3L ) == 0 ) ? 0 : ( 4 - ( tokenData & 3L ) );
						break;
						
					case kSkipPixelsToken:
						destPtr += tokenData;
						xCount += tokenData;
						break;
						
					case kLineStartToken:
						// if this line is above the clip rect, skip to the next line
						if( yCount < clipRect.top )
							{
								srcPtr += tokenData;
							}

						// set up the destination pointer
						destPtr = rowStart;
						rowStart += sRowBytes;
						
						// move the yCounter
						yCount++;
						
						// reset the xCounter
						xCount = 0;
						
						// if we have hit the bottom clip, exit the loop
						if ( yCount > clipRect.bottom )
							{
								exitFlag = kTrue;
							}
						break;
						
					case kEndShapeToken:
						// signal a loop exit
						exitFlag = kTrue;
						break;
						
					default:
						// we should never get here
						Debugger();
						break;
				}
		}
}


//
//	drawSpriteXXX -
//
//	Sprite draw functions compiled by the shape compiler
//

void drawSprite201( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 3.53326534946038615300e+30;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x46464c4c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4c4c5151;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5151;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04883368754677094700e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.38145999623298654900e+83;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e4646;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4c4c5151;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x51585158;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x51585158;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x58;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04155764666480188300e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.90890922887208027300e+117;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x58585858;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e3e46;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x464c4c51;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x586f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f5f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x58585858;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5858;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x58;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04155764666588549000e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750260809301467500e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e3e46;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x464c516f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5f585858;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x585f;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04155764667223823700e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750279024918258300e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.06643567162987024900e+272;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f5f5f5f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e3e46;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x464c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f788282;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7878726f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x67676f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f5f5f5f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5f67;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x67;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04155773183631888600e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750279069199210100e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.15581740169086879100e-253;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750260875748850800e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e3e46;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4c6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f828b8b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b828278;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f676f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5f676767;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x67;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04885549145514838800e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750279109119551400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e464c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f8b9797;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30520187705414353700e+190;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f67676f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x67676f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f67676f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04885549145514838800e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750310791130377000e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e464c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b979da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 8.75376359309405519300e+189;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x675f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f6f;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04887325502354239000e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95758413452084121100e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e4651;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f8b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9097a3a3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.99965655664714301000e+151;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5f586f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f6f;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04885549145514838800e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.40583987775500161400e+229;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e464c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f828b8b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x90979da3;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 3.85119844093700367700e+117;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x58586f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04885549145515583300e-09;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x82828b8b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e464c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f78;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x82828b8b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9090;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 5.29234429830612174200e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984928702200e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x51516f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f7272;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x72;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04885549145515583300e-09;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x82828b8b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e464c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f78;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x82828b8b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8b;
	destPtr += sizeof( unsigned char );

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 4.37772459851438120800e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876992900e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4c516f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f72;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7278;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04155821755889215900e-09;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e3e46;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7882;
	destPtr += sizeof( unsigned short );

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876992900e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f72;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7878;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04897983643390640500e-09;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e466f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7878;
	destPtr += sizeof( unsigned short );

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7878;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x78;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04897983643390640500e-09;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e466f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f78;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x78;
	destPtr += sizeof( unsigned char );

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x78787882;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x78787882;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.03835833108271738900e-08;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.98757999307342775400e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x67676f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e464c6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f726f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f676767;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x67676f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x78;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f788278;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f788278;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04897983643390640500e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278949437647900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x5f5f676f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e466f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f67675f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5f5f676f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7278;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f7882;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f7882;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x82;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.03835833108271738900e-08;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e464c6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7882;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f8282;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8282;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x82;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 7.04897983643390640500e-09;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e3e466f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f82;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x82;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f82;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f82;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x82;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.03835833108271738900e-08;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e464c6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f82;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8b;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f82;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f82;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8282;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.04472479399512788600e-08;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3e466f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8b;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f82;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f82;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x828b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.55501425681651956200e+30;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x46466f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x90;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x828b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.53335030612902540700e+30;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x46464c6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x828b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.53335030612902540700e+30;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x46464c6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x828b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.50575220698769161300e+30;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x464c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x828b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.50575220698769161300e+30;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x464c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.50575220698769161300e+30;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x464c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x90;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x90;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.37772459851438120800e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4c516f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f90;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.37772459851438120800e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4c516f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f90;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.26507294544894903200e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5151586f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8b97;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.26507294544894903200e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278985361632400e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5151586f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f8b90;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9097;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 7.41713839917801291300e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x51586f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x90;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x979d;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.84134793938790505800e+117;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x58585f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f8b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f90;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f90;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x97a3;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.83703834496017441100e+117;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5858586f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6f90;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.56982468104226037200e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f90;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c4c6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f90;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9da3;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 3.85119844093700367700e+117;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x58586f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9097;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.37772459851438120800e+59;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f97;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4c516f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f97;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 3.84134793938790505800e+117;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x58585f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9797;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.29234429830612174200e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f909d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x51516f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f909d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 3.84134793938790505800e+117;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x58585f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9d9d;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.41713839917801291300e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f97a3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x51586f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f97a3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 2.56994257661654897600e+151;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f979d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5f5f676f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f979d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.94952936366149978500e+117;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f90a3a3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x585f676f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f90a3a3;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 2.56994257661654897600e+151;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984878676200e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x90979d9d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5f5f676f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f8b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x90979d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x97;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.56994257661654897600e+151;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x909da3a3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5f5f676f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x909da3a3;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30346144407928155400e+190;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.40316437057009559100e+229;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6767676f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f828282;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b8b8b90;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9090;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x90;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30520188701298090200e+190;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984879484100e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x67676f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x726f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f97;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3a3;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30346144407928155400e+190;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.40316437057009424400e+229;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6767676f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f828282;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b8b8b8b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x97a3;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 1.75075271861881103900e+190;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x676f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8282;
	destPtr += sizeof( unsigned short );

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = 1.75075271862049288500e+190;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x676f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f90a3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3a3;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 5.95750278984877396900e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f78;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x82;
	destPtr += sizeof( unsigned char );

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = 1.75075271904907017100e+190;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x676f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f909da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.95751146522120639800e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6f6f6f72;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f7878;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = 1.75075281216709690300e+190;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x676f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b9da3a3;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.95973236056286995700e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95153395292677763000e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.29482191369511656000e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.34818540140308317900e+59;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f726f8b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6f6f7272;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f787272;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f675f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5f585851;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x514c4c4c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4c4c4c4c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4c515151;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x58585f67;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f726f8b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x97a3;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 6.99203920245619810600e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 8.73626115024661165400e+189;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.25068166887864702000e+83;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.26496612668123978200e+83;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x828b909d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6f727272;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x72726f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x675f5f58;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x58515151;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x51514c4c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4c4c5151;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x51515858;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5f676f72;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x828b909d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.97059432553108578700e+243;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984878676200e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x72727878;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f8b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x909d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 2.06842843973350627400e+272;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750300571505386600e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x78787878;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x726f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x828b909d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 2.06842847013988636900e+272;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6f828b90;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x78787878;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x78786f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f828b90;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9d9d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 3.12913522290406023400e+272;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984876790900e+228;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750310737955234000e+228;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x78828278;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8278786f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b8b9097;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9d9d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -1.41512921789466268300e-296;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.95750278984878676200e+228;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x8b90979d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x82828282;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x82828282;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f6f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6f6f6f8b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b90979d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9d9d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = -1.41781705828934687800e-296;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.69966633044341041900e-253;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x82828b82;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b8b8b8b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8b8b9090;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x97979d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = -4.69632325564425271100e-253;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8b8b8b8b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x90979797;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x979d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

}


void drawSprite202( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72250940029240117099e-294;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3030505;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5050505;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x505;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72250942371548806130e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 8.67484387523436138642e-275;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3030505;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7070707;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7080707;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7070707;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 5.67893202851098500610e-299;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.19344451504962361539e-265;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x8080807;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2030405;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7080809;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x90b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b09;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8080807;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x707;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 5.67777456296018666253e-299;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.93479391210433230336e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.73442238014412008017e-246;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030307;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x709090a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0d0d0e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe0e0e0e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd0b0a0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9080808;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x808;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20788743429225237844e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.43427612470573747663e-226;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.93058417461684816889e-197;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb080808;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1020305;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8090b0e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x11171117;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x17111717;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x17171711;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe0b0a0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb080808;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x808;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20788743450695324194e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1020305;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x80b1117;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1f1f;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x1f;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.92764088186278782542e-197;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9090909;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x17170e0e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa0a0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9090909;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20790142302401441388e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1020307;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0e1724;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2a2a;
	destPtr += sizeof( unsigned short );

	destPtr += 13;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25629270638231956133e-197;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9090a0a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x170e0d0a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9090a0a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75219239009344923277e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1010305;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0e1f2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x36;
	destPtr += sizeof( unsigned char );

	destPtr += 17;

#ifdef powerc
	*( ( double * )destPtr ) = 5.06882535264059285210e-241;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe0b0a09;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa0b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75219238977516119680e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1010305;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b1f31;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x808090b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x808090b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.64579535421898308123e-260;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa0a0909;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb0b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75041233068502848459e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1010205;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b172a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x3a;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x6050708;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6050708;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x90d;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.88210303732048639916e-265;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x909090b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd0d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75039842396842157845e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1010203;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b1f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3142;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.59814890255287803343e-284;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5030305;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8090e11;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68700869930303261588e-270;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x808090b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd0e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 7.74861836488158343826e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1010103;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2a36;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa0e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x17;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68608443978258982314e-270;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xd0e0e11;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x808080b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd0e0e11;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 7.74861836488158343826e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1010103;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb2a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x31;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72711456683779522578e-294;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb111f1f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.31946835496606942622e-275;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7070b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe11;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x11;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75045405081485187940e-304;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x101020b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb2a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x2a;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549913346859703e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb17242a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.85296625825415912806e-284;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb111117;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5060b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb111117;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75045405081485187940e-304;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b1f24;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x101020b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b1f24;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549830070301984e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b2a31;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.69913535242243710705e-289;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb111717;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4050b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb111717;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x17;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20792923612393248376e-304;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b1717;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x102030b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b1717;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.11855369938185352617e-294;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3050b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b313a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.92283413296793037301e-294;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b1117;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3040b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b1117;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x17;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75045405081485187940e-304;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b11;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x101020b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b11;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x110e;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68012602116193842723e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x203050b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b2a3a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72711456656080412516e-294;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b17;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b17;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1f1f;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20792923612393248376e-304;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x102030b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe0a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.67894121382533618935e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030407;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b36;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4242;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549829224262262e-299;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b17;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b17;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1f1f;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75045405081485187940e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497281075225e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.97252709362450942983e-270;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b42;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x101020b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0a09;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8050403;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303070b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b42;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x47;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549829224262262e-299;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1f24;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20792923612393248376e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497275017998e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.44667947681654911451e-270;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b3642;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x102030b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0909;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8070505;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5070b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3642;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x47;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549829224262262e-299;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1f24;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x2a;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 8.22216970881837857261e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb314247;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1020b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb314247;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x47;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20792923612393248376e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070528452347176e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x102030b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb1f242a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549829224262262e-299;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb3a4247;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb3a4247;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 8.66363826905782299703e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497475687310e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x103030d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b2a2a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.37914850418618039256e-299;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb364247;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x202030b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb364247;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75223410990165764050e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287771300e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x101030b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b24;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x2a;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.67779286210721043383e-299;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x31424242;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x203030b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x31424242;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75039847829148144807e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1010203;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2a2a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x31;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549829224262262e-299;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497288078894e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b31;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3a42;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75039842396838919936e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb242a31;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1010203;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb242a31;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72711456656080412516e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497518371833e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3136;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3a36;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75039842396838919936e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b2a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1010203;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x31;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72711456656080412516e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070556443077968e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb313136;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x31;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 7.75039125332021236952e-304;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1010202;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb31;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x36;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72711456656080412516e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107085641167564645e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3131312a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.07930098652778746929e-299;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497518466477e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2010203;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x40b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b313a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72711456656080412516e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b2a2a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b2a2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3131;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x2a;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 3.52449024927218181244e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3020203;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x313a;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.92283413296793037301e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb2a2a31;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3040b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb2a2a31;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2a24;
	destPtr += sizeof( unsigned short );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 2.43857774665491140315e-289;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b3a42;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4030303;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030d0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3a42;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.11855369937505662087e-294;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x2a2a312a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3050b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2a2a312a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.59814631224332939767e-284;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80108294772878079102e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.44448755083874646704e-241;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b3a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5030303;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30d0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b17;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x171f1711;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe0d0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b3a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.69713118406242813427e-289;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287913267e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x405070b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2a31;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x2a;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 7.58889557276947001908e-275;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070528452086904e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.42597171163697127428e-197;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7050505;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x50b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb1f241f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x17110e0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3142;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.76726352984304903198e-284;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287913267e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x505060b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x312a;
	destPtr += sizeof( unsigned short );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 5.44853524506146092379e-270;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497475687310e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8070707;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x70b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b2a2a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.73447078766311508211e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b3a42;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xb0a0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3a42;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x47;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.93702717194474987365e-284;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070545588408532e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5070b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb2a3131;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.54377340997485654670e-260;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497288078894e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa090808;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x80b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b31;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3631;
	destPtr += sizeof( unsigned short );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 2.54496897942101801524e-260;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b4242;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xa090b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b4242;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.26880453958856754027e-279;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070556473245798e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x607080b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb313631;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.06882821308526113519e-241;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497288291843e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe0b0a0a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b3a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x423a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x31;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72824946661387023245e-265;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb364250;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9080b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb364250;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.26858934364951962796e-279;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x607070b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2a36;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x3a;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.42597556905913650337e-197;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 8.85451595357130745708e-159;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x17110e0e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x11111717;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1f1f1f24;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b42;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4242;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x3a;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.45224678111056325917e-270;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb3a4250;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8070b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb3a4250;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.31946835496606942622e-275;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b0b36;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7070b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b36;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4242;
	destPtr += sizeof( unsigned short );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.93059191453505759257e-197;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.42602583891454175742e-105;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x17171717;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1f1f242a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2a2a2a2a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x313a4242;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4742;
	destPtr += sizeof( unsigned short );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 1.26945012740571127719e-279;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6070b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb42;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.31523743064763623191e-275;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b313a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x707080b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b313a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x241f;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x1f;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x31313642;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x31313642;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4242;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.76890534456356838248e-284;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5050b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3a47;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.31523743064763623191e-275;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb0b3142;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x707080b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3142;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
#else
#endif
	*( ( unsigned char * )destPtr ) = 0x2a;
	destPtr += sizeof( unsigned char );

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 2.43959554980127513230e-289;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb425050;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x403050b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb425050;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68700869930303261588e-270;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x808090b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb36;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72252738922313710373e-294;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x303050b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3a50;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68700869930303261588e-270;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x808090b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb36;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68712549829228294054e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b31;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4250;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.88210303732050402801e-265;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x909090b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3131;
	destPtr += sizeof( unsigned short );

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72711456656084236862e-294;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3030b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b42;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5050;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.88210303732050072260e-265;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x909090b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b24;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2a2a;
	destPtr += sizeof( unsigned short );

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 6.28441421414713229173e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2050b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3a47;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.03838659722032185717e-265;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x90a0b0e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b1f1f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68245918021146434795e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x203070b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3642;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5050;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xa0b0e11;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xa0b0e11;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb17;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x17;
	destPtr += sizeof( unsigned char );

	destPtr += 26;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68011235031006155520e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030508;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b3142;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5050;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xb0d0e11;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xb0d0e11;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1111;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.59814631204699581846e-284;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5030303;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030303;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x3;
	destPtr += sizeof( unsigned char );

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 5.68011235030816024697e-299;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2030508;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b2a42;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5050;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.59703838769731764529e-246;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.58606389481079225109e-275;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.76659650237228117973e-284;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.72097434486960227227e-294;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x80b0b2a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xd0e0e0e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd0b0908;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7050303;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030305;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5050403;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030303;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030303;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030306;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x80b0b2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3a47;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.63415218370834552088e-241;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.59739060462013118246e-275;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.76692744071187187329e-284;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.11318401535359579459e-294;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb2a3a47;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe0e0e0d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa090807;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7050b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b06;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5050505;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5050304;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3050405;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x608090b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb2a3a47;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 5.44668437234347639520e-241;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287913267e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe0d0e0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b2a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3a47;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 5.63415219496976549874e-241;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107082862506134355e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe0e0e0d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2a314247;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.80107573659384415010e-226;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb2a313a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x11111111;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x11110b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb2a313a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4250;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 1.93059187450954259104e-197;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497287179774e-255;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107082862433329323e-255;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x17171717;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1717170b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2a313642;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4247;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 8.85449458743897586560e-159;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.80107070497288078894e-255;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x313a4242;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1f1f1f1f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1f1f1f24;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b0b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb0b0b31;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x313a4242;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4247;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x50;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.79989762568828742774e-134;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.75039198497925523990e-72;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x242a2a2a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2a2a3131;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x31313a3a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x42424247;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x47;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 1.87401829425082374350e-105;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2a313136;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x363a3a42;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4242;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x42;
	destPtr += sizeof( unsigned char );

}


void drawSprite203( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 5.96886837358847964068e-212;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x14141818;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x18181818;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1919;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.66530005302294431320e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.92859137247324165270e-168;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12181418;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x18191919;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1d1d1d1d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1d1d1d1d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x1d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38974180141845178324e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.57503346064143469493e-125;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x26262026;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1214181e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x191d2626;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303426;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26262026;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2020;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x20;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 1.45889786211594422281e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39804855727469229030e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39804323441784797427e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12151819;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x20262734;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26262626;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2026;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x26;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25033331656924100781e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.25317511168418752188e-23;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.03878450345464502166e-13;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30262626;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121419;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26273430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3b3b3d3d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3d3d4444;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3d3d3d3b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30262626;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2627;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x2c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25141818342782465883e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.35081296829431404907e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74348998178976257538e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1212181d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26303b3d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x494e4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x443d3b34;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x27272c27;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x2c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25142772268920836623e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121826;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303b444e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5555;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x4e;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.36342330610401054694e+69;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3430342c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4e49493d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3b343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3430342c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2c2d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x30;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25142772272204632275e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121826;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303d4e5b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x60;
	destPtr += sizeof( unsigned char );

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = 5.01822645364069498880e+20;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x302d3030;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x443b3430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2d303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x302d3030;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x30;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25034074537320443315e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121420;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303d4e60;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = 2.57705804236040794270e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x2d303434;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x34302d2c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2d303434;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25060774074804462026e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1212151d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303b4e60;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x26262c30;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x26262c30;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.32194614772159748805e-91;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343434;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2d2c2c34;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343434;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24979303528228364581e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121219;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3034495b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x19192026;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x19192026;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2c34;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.28950355815635820633e-120;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3434343b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x27262734;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3434343b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24979197996289349470e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121218;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3034304e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x60;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.32024368996347889783e-192;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x18181819;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x262c343d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 6.54409091235446407644e-125;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343b3b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x26262630;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343b3b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x3b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24978777511655882552e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121214;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4e5b;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.70876669810401539576e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15183030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2d3b4449;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 6.04269565062647683744e-154;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x343b3b3d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x20203430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x343b3b3d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24982152886168962563e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121234;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4e55;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.39732541056402093421e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12143430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303d494e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.04816936186956171800e-187;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30303d3d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x191d3034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303d3d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x3d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24982152886169060847e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121234;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3049;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x49;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.67289421932564587188e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12183034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030555b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38105142865811951279e-192;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303d3d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x18193430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303d3d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4444;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25035764673659127899e-221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3444;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x3d;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.48669479633730101537e-212;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x14193430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34305b60;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.18502596193770829923e-212;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343044;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x14183034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343044;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4449;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24982152879752974531e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303b3b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121234;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303b3b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x34;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 7.16644255493426953719e-212;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x14182030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303460;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.70876677882597870934e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15183030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4449;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x4e;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25035764673634065446e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74484468511033766599e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.32024785084794888679e-192;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34606565;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343434;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2d272619;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1818181e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x20273430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34606565;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.67397064355841941888e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12183430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4449;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x4e;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25062780809708044234e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529931689769573e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.67975472950682138836e-154;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34606565;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121530;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x302c2726;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2019191d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26273430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34606565;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25143829217829730784e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529948083441745e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121830;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30344e4e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25036185158266893516e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39939270802256690725e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142387651628466385e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x5b656565;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121434;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5b656565;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.39138186048675357588e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435455280416e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12141e30;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34304e55;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x55;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25900281027224450607e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.57894439386651186520e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529939836563587e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12123430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030345b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6065;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25143829217829730784e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39939270802257457771e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121830;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303449;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5555;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.39624898624681714213e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004888961118530e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58581182681921845996e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12143034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303b3d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x494e5560;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6060;
	destPtr += sizeof( unsigned short );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.10020570106997638292e-192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39939268751245418201e-76;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30345555;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x18141418;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x19343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30345555;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25144249702462558855e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529939835244268e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58147273218211323242e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x12121834;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303444;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x494e555b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5b5b;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 3.55809299341230064616e-149;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142387651628239993e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x211d1e19;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x191d3430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x555b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.45892221703422538228e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435018740086e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343044;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12151830;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343044;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x49;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.57330955233975257594e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39939270802256690725e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004888962253757e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x34302726;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26212626;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303455;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5b5b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.39732541041751615234e-221;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529947846884804e-76;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303444;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12143430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303444;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x44;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 4.32192738415427343596e-91;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529947846884804e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.57894439509680227535e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2d2c2c2c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2c2c2c2d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30555b5b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.70877861423489973633e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435076470040e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3034303d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15183034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3034303d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x3d;
	destPtr += sizeof( unsigned char );

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 3.21617143471033017149e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142387651628239993e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34305b60;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x34343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34305b60;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 7.18500790226024362792e-212;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529947847007531e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39940720362485433412e-76;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x2c272727;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x14183030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030343b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3434302d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2c272727;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x2c;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.57892737780911146364e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004888929823059e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3430302d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6060;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 7.18964916854748024826e-212;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435076470040e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.57894439360628051345e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x26262626;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x14183430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x302d2c27;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26262626;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2c2d;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25986393811823836542e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004888931173059e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x302d2d2c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303060;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 7.18502596166213949026e-212;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529947847007531e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004888961118530e-76;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3030302c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x14183034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030302c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2c30;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x3b;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 6.59475217341444107613e-96;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004889308344847e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2c2c2c30;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34306065;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.65417142175701492011e-192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39939270802256690725e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142387651628466385e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349002852127798842e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x181e3030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303b3d44;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 6.54526299767510712635e-125;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x26262734;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3465;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.32625618191954036475e-192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.57894439386651186520e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529939835244268e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391476110978032e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x18183430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x343b4449;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.06681433350874635151e-125;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x26212630;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3065;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.37613175074957057719e-192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004888961118530e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435018740086e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529948044168998e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x18191d34;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3034494e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x55;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 6.04269565062647683744e-154;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303460;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x20203430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303460;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b6b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 9.02280348318524316315e-188;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39939268751537049034e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 8.47367070262480683990e+102;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435455280416e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x19192030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30345555;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x554e443d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34304e55;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x5b;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.04816936186956171800e-187;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343065;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x191d3034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343065;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b76;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.04872828178186977382e-187;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39944166741143658069e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.06616638376866776847e+102;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435644261118e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x191d3430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303455;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5b60605b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x55493d3b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34305b60;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38105142865811951279e-192;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303465;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x18193430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303465;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.13955134911188362532e-168;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303060;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1d202630;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303060;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x656b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 6.51267843245000356758e+44;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x493d3430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2d303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3460;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.32540335082823690946e-192;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3034606b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x18183034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3034606b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.13645427723622233090e-168;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30345b65;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1d202034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30345b65;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.21651969127464606919e-76;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30343065;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x302c2c34;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343065;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.18964923909309534463e-212;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34306576;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x14183430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34306576;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.27439371342953344670e-154;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30606576;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x20263034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30606576;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 6.55565009862011607911e-125;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30656b6b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x26263034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30656b6b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.67289421938772040867e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x30346b76;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12183034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30346b76;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 6.02230175018824990732e-154;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x20202630;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3060;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.42269180562000360172e-149;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34656b76;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x21263030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34656b76;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.67397065998359940588e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3465767b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12183430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3465767b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.25980388098526270604e-154;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x20262630;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3460;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.14679671801650384134e-168;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1d203430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x306b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25143829217854793237e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34306b7b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121830;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34306b7b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.41411465573755779391e-149;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x21262734;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5b60;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.16266300994449942587e-187;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x19203034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b76;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25144249702462558855e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3065767b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121834;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3065767b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 6.54524503468168729341e-125;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x26262730;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x555b;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.93964711818581827622e-168;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1d263030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303065;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7676;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.39732542684269613934e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x65767b7b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12143430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x65767b7b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.45505118842565174114e-120;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x272c2c30;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3430344e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4e;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.13955138057630431451e-168;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1d202630;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303465;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b6b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25143829217854793237e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x346b7b7b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121830;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x346b7b7b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 6.59475217341445438225e-96;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2c2c2c30;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343044;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4949;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 8.26856688735137638940e-154;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x20262c34;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x304e5b65;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b6b;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25060353578734899407e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34657b7b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121519;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34657b7b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 6.83250783814632628089e-96;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2c2d3034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343d44;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x3d;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 8.33307114620039148043e-125;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x262c3444;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4e5b6060;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24979303521654065386e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3465767b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121219;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3465767b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.39939795861229194545e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x30303434;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343b3b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x34;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2c343d49;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2c343d49;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4e;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.24979197996288612339e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3465767b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121218;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3465767b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.74484470095627129210e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x30343434;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x303b3434;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x2d;
	destPtr += sizeof( unsigned char );

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25060351936216900706e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3065767b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121519;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3065767b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.21868738048803367239e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3434343b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3034302d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2c26;
	destPtr += sizeof( unsigned short );

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = 1.25060236916648384463e-221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x34306576;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x12121518;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x19303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34306576;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7b7b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 3.21868738048803310641e-57;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x27262620;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3434343b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3034302c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x27262620;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.31938663234659431606e-192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142438224035824804e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x18181418;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x18181d30;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x65767b7b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 4.33820782882688425350e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.18472699911896261683e-96;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.00906001848791955717e-154;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.50571192813110647614e-153;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x343b3b3b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3030302c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2c262626;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x21262026;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x20201d19;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1e181919;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x20343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343065;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x767b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 2.25317510157184573016e-23;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.58922997528924903166e-96;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.01347015176062219316e-154;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349554767267491442e-76;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3b3b3d3d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3b303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2c2c2626;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26262126;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x20202020;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x26303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x606b767b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.03878561525983505293e-13;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.39804328617572067503e-76;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349004888961118530e-76;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x305b656b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3d3d3d3d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3d3d3034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30303030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343030;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x305b656b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x767b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 9.32896043071468208128e+20;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.58142391435076470040e-57;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.57894474684227950937e-57;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x44494944;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x443d3d30;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303430;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x34303034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5560656b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x767b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 1.35081298171157675020e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.74349529947848020032e-76;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x5b60656b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x494e4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494444;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343034;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x30343055;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5b60656b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7676;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.63409572572377894974e+69;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.21362811930563624922e+132;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4e4e4e55;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x55555555;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5b5b5b60;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x65656b76;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53181402612249651328e+103;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x555b5b60;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x60606565;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b6b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

}


void drawSprite204( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 3.81573873114519058028e-236;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf0f0f10;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x13131616;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1616;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22468127662645578457e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.58625146525953822635e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0f0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf101316;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x161c1c1c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1c1c1c1c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x1c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22416978925123495697e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.28582968683271998973e-202;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x2b25221c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf0f1013;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x16323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232322b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b25221c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1c1c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x1c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22416779126400190212e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930044771550924726e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c0c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf101332;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b25221c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x1c1c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x22;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22416779129571952078e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060361292230204e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.78043573956212450303e+127;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x25221c22;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c0c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32324d4d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5a5a4d4d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x43322b2b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x25221c22;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2525;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x2b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22416979220983065123e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060361733684743e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.27457195486433490455e+199;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.57089046955006958426e-101;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x10323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32325a63;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x696e6963;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5a433932;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b253232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2525252b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x2b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22416988066284582825e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060361735134295e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32325a6e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d7d;
	destPtr += sizeof( unsigned short );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 7.57089074740053093975e-101;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2b253232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32322b2b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2b32;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22416988066284582825e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360386655401e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323275;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20332046310619875586e-130;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x25223232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3232;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x32;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22417054666201892582e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360386655401e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c10;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323275;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.85004171175845888828e-173;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323239;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1c1c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323239;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22417054666201892582e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360385074071e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c10;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323269;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 3.59725837544867181477e-202;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323932;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x161c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323932;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x39;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22417054666201892582e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360384283406e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c10;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x757d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 2.83177780320437862804e-202;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323939;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x16163232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323939;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3939;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22417054666201892582e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c10;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6975;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 1.00604957367678289445e-216;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323239;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x13163232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323239;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3939;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x39;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22416988066284582825e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0c0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x636e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x75;
	destPtr += sizeof( unsigned char );

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3939;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x43;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22470467799884317468e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360384283406e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0f32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x696e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x69;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
#else
#endif
	*( ( unsigned char * )destPtr ) = 0x4d;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060507768427532e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32434343;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22470467799884317468e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360383097409e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0f32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232325a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6363;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x63;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
#else
#endif
	*( ( unsigned char * )destPtr ) = 0x43;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.71667468907643731803e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060421406741981e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32394343;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22487517378715615187e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360384283406e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c1032;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6363;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x4343;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360954879870e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232434d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22470467799884317468e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360384283406e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0f32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x635a;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x3939;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x39;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.71667468907643731803e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360381384302e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232324d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4d4d;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22538666115209508345e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060362037300044e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c1332;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32326363;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x635a;
	destPtr += sizeof( unsigned short );

	destPtr += 3;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x32323943;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x32323943;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360381384302e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232324d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4d5a;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22470467799884317468e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060361733684743e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0c0f32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32325a63;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x635a;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2b323943;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2b323943;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.71667468907643731803e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5a5a;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.35581593921152263535e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060362037300044e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0f1032;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32326363;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x5a;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x22253239;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x22253239;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4d;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4d5a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5a;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.36161279601416385988e-250;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060785209559243e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc0f3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32636363;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x1c222b32;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1c222b32;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x435a;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.71667468907643731803e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5a5a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5a;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.83259930047017971284e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930169117306728351e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf0f3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6363635a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 3;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x1c1c1c2b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1c1c1c2b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x324d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5a;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x325a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5a;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.81628259913499524805e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930169117306728351e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf0f1032;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6363635a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x161c1c32;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x161c1c32;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x325a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x63;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.71667468907643731803e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x325a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5a;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.81772230807633505377e-236;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xf0f1332;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6369;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x63;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81783735730185550308e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x16161639;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32325a69;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x325a6363;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x325a6363;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.97954371111045679112e-236;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323269;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xf103232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323269;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6963;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.23247603271438981693e-173;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c16161c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6e;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x325a6363;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x325a6363;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.97954371111045679112e-236;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3232696e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xf103232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232696e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6e63;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.23488980845127902029e-173;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c161c39;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323269;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x75;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.00604957367678289445e-216;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32326369;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x13163232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32326369;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.71667468907643731803e-236;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32637575;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xf133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32637575;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6e;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.25115351847662107740e-144;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x221c1c32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x757d;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.83526632143726277668e-202;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3232696e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x16163932;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232696e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32757575;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32757575;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 7.55271436843778702464e-101;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2b25252b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x327d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.67849611754082032325e-173;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32326975;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1c223232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32326975;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.09111992423313396074e-231;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x10133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6975;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.61625398369836989928e-33;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32327d7d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x392b2b32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32327d7d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.35590471061797789969e-144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32326e75;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x222b3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32326e75;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.00604957367679078486e-216;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x13163232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x757d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 4.85787517542199504781e-33;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x757d7d7d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x39393939;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x434d5a69;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x757d7d7d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 9.72385186057170865630e-101;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.50443723213991137885e-33;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32327575;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2b2b3943;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x43434339;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x39323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32327575;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.00604957367679078486e-216;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x13163232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x757d;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 4.85787517542199094245e-33;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x69757d7d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x39393939;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x434d5a63;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x69757d7d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.80189694820340764537e-100;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3232757d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2b393943;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x43433939;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232757d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.82081388875819351327e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x16161c32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6e;
	destPtr += sizeof( unsigned char );

	destPtr += 34;

#ifdef powerc
	*( ( double * )destPtr ) = 9.71399210605707941009e-101;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x326e7d7d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2b2b3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x326e7d7d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.82081388875818943397e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x16161c32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232325a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x63;
	destPtr += sizeof( unsigned char );

	destPtr += 34;

#ifdef powerc
	*( ( double * )destPtr ) = 3.39490482493008945597e-144;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32757d7d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x22253232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32757d7d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.59725837545182284735e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x161c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32324d5a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 34;

#ifdef powerc
	*( ( double * )destPtr ) = 2.85004171175845888828e-173;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c1c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3275;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.58928461948977403424e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x161c2232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32324343;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 34;

#ifdef powerc
	*( ( double * )destPtr ) = 3.59725837544867181477e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x161c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6e7d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.84372425142792961551e-173;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c1c2232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323939;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 34;

#ifdef powerc
	*( ( double * )destPtr ) = 2.83177780320437862804e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x16163232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x757d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 2.85004171192319950406e-173;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.02016346790597628862e-101;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.00107266635959885773e-216;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c1c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32393232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b1c1c16;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x13131313;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x13161616;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1c252b32;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x39;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.81573682711813030151e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.59725837544867181477e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf0f0f0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf0f1013;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x161c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 2.84490877523937643960e-173;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 8.16438397269419668365e-130;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.81778078060424757596e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c1c2532;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232322b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x25221c16;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x16161616;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1616161c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x222b3239;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x43;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.81573683455562295508e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.64291484341642707669e-202;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf0f0f0f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x10131316;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x16223232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323275;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 2.84490877523937895334e-173;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360381384302e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c1c2532;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232324d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4d;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.83259930047018276812e-236;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360387709621e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf0f3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323239;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232327d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8787;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 3.39052526815937642270e-144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060361733684743e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x22252b32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32325a63;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.70072908683366120881e-217;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060362647956860e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x13133232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232757d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 3.38677136235590810846e-144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060837399105494e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x22252532;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32696e6e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.59725837544867181477e-202;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060881089268207e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x161c3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x326e7d87;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22484544133499264567e-129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x325a636e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x252b2b32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x325a636e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7575;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x75;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.66820856727347385292e-173;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930061010969146868e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1c22252b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x327d8787;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 9.70422536809446231429e-101;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060508108808748e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x5a63696e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2b2b2b32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32434d5a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5a63696e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6e6e;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.27468475906630811640e-173;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1c25252b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b323239;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 9.71399210605707941009e-101;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930120400376293762e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2b2b3232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4d5a5a5a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6363;
	destPtr += sizeof( unsigned short );

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = 9.70422551712305671458e-101;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x757d8787;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2b2b2b32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x39323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x757d8787;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3232324d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232324d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4d;
	destPtr += sizeof( unsigned char );

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 9.34574624210637650120e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x32393232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6e7d8787;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 6.74934022219974812997e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3232434d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x32323239;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3232434d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 22;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x32393263;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x32393263;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 9.34578452768094309998e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x32393239;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3243;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x43;
	destPtr += sizeof( unsigned char );

	destPtr += 22;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x39435a75;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x39435a75;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d87;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.85787497905379971406e-33;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.09976205316150000000e+15;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.23246196407473310063e-173;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.00107266632793932330e-216;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x435a637d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x39393939;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x43393932;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b2b221c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1c161613;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x13131313;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x13161616;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1c222b32;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x435a637d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 4.85790436641258190312e-33;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.85256747183677308655e-33;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.81776900841820860642e-202;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.84370415888672142959e-173;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x39393943;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323243;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3939322b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x251c1c1c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x16161616;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x16161616;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1c1c2225;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2b39434d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6375;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 7.41973951755184147038e-33;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930120400683467055e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x39434343;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x43323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4d5a6375;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.08660379231448040000e+16;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x324d5a63;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x43434d4d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4d433232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x324d5a63;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6975;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 2.41082550045101854392e+64;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360377826310e-67;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930149141850837080e-67;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4d4d4d4d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4d4d4332;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5a5a6369;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7575;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 1.78044920803678797525e+127;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.74930060360384283406e-67;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x63697575;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5a5a4d5a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4d4d4d4d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323232;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x32323263;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x63697575;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d7d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.78388675187454530589e+127;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.67819592817327473730e+170;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5a5a5a5a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5a636363;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x63696e75;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x757d7d7d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x87;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 5.86074173878328809529e+170;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x63636969;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6e757575;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7d7d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7d;
	destPtr += sizeof( unsigned char );

}


void drawSprite205( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30022647317762292130e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d777c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7c7c8080;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8088;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.29849945220792627884e+219;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.93746833887567805082e-306;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d6d77;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x777c7c80;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x80808088;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88889b9b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xab;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 3.02669729692702308216e+267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.20531975766017409087e-266;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xabb6b6b6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7777777c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7c808080;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88dadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaab;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xabb6b6b6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb6b6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xab;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 5.14613741798719274117e+291;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032731977825e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7c808080;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x888888da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadad4d4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc6b6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xab;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 5.15592145276589322175e+291;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032827342993e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdddad4b6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7c808888;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xababdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadae1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdddad4b6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb6ab;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 5.17923388494547428164e+291;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.74182857225351840830e+144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580097636316876e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.54734290117632816528e+192;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7c809bab;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd4dadadd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdde1dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaeaeaea;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe7e1ddd4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc6ababab;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xc6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 4.30039049453252154326e+267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.40841807244228833325e+192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.60765014586479100827e+221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7780abc6;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae1e1e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe7e7e7ea;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeaeaeded;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xedefefef;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeff0f0ef;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xedea;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe7;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xabababb6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xabababb6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xc6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x6d809bc6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d809bc6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe1;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -3.38084652480603769186e+221;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xf0f0f0f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xedededef;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeff0f0f0;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf0f0f0f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf0f0;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xef;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xabababb6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xabababb6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc6d4;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x6d7c88c6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d7c88c6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdde7;
	destPtr += sizeof( unsigned short );

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = -4.85828612826474685932e-176;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9b889bda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb6d4d4da;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x6d6d80ab;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d80ab;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdae7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xed;
	destPtr += sizeof( unsigned char );

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = -4.38051326685801221422e-306;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x80889bda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdad4dada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xda;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30110941433129283246e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7c9b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae1eaf0;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 24;

#ifdef powerc
	*( ( double * )destPtr ) = 5.14656720902629915541e+291;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7c8080da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdadd;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xdd;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30023480094263410905e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7788;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadae7ef;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = 3.67630819244036935474e+267;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadddde1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x777c80da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadddde1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.29850577323072905719e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d6d80;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaea;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 24;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30184128852752919118e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadde1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d80da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadde1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30029001132853473578e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d77da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaed;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30109123522905560551e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7c80;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe1e1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.29856636999631238622e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d6dda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xeff0;
	destPtr += sizeof( unsigned short );

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30022941456294591999e+219;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580061181026652e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7780;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae1e1e1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe7;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30115183199464144277e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7cda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xeff0;
	destPtr += sizeof( unsigned short );

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.29849271927885359273e+219;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdae1e7e7;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d6d6d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7780dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae1e7e7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30029001132853175725e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d77da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdaf0;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30022647321871773304e+219;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.93465422161713583546e-306;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.00027213925143523803e-306;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032827342993e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d777c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7c808080;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x80807c7c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7777777c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8080dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadae1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe7e7;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30115183199464144277e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7cda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xedf0;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 3.02669732791899465685e+267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.48603973794539295350e-267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7777777c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x80888888;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88888888;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88808080;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe7e7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe7;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30029001132853175725e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaeff0f0;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d77da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaeff0f0;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.44438389470194940113e+291;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7c7c80da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdae7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xea;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30115183199464144277e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7cda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdaef;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -4.42431940869604412321e-306;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdae7eaea;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8088dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae7eaea;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.10520550377721628024e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d77dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdaef;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -1.49439716823573297522e-267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.29780235924270782328e+163;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.30991017601494561169e+144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadae7ea;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8888abda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd4dde1e1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1e7e1e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1e1e1e1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdddadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadae7ea;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xed;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.54645768482537526414e+219;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6d7cdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdaef;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -2.53022815582747093252e-98;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.40841152022363949860e+192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.21803551398352980077e+163;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadaea;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xabababd4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdde1e1e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe7e7e7e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe7e7e7e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1e1e1dd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaea;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xed;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30184128852752919118e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaefef;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d80da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaefef;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xababc6da;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xababc6da;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdde1;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -3.40505678434169214592e+192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe7e7e1e1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1dddada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xeaed;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.07417995979726760169e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaeaeded;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d7780da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaeaeded;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0xabab;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb6;
	destPtr += sizeof( unsigned char );

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = -3.21521545510474814630e+163;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaeaedef;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe1e1ddda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaeaedef;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72165207576614628667e+267;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdae7eaea;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x777cdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae7eaea;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
#else
#endif
	*( ( unsigned char * )destPtr ) = 0x9b;
	destPtr += sizeof( unsigned char );

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = -1.45624213931100774753e+144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xdddddada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xeaed;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xef;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72165207576614628667e+267;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x777cdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdae7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe7;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xabababab;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xabababab;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc6c6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd4;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -4.64965840711277086698e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xdadad4d4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xedef;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72165207576614628667e+267;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x777cdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdae1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe1;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -4.36713027270843606404e-306;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8088889b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xababb6c6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdada;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.49091608334559367673e+100;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032828204912e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xd4c6c6d4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaef;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25620199165454914506e+291;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdae1dddd;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7c80dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae1dddd;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xda;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 5.14617549159718339540e+291;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xabc6d4da;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7c808088;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88dadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xabc6d4da;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe1;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -4.60700269169415580012e+32;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580033159243330e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc6b6b6da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaeff0;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25620199165454914506e+291;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65576650038407614635e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7c80dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaddda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd4ab9b88;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = -2.93744661304752670400e-306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x80808080;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88dadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdde1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe1;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.53416552997450524988e-98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xababb6da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdaef;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = -3.00027213925143523803e-306;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65371972694506598748e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.48414715177365853340e-267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580045042334308e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8080dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadad4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc6b6ab9b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88888088;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadde1e7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.53023651081697242963e-98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xabababda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdaf0;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = -3.00027213925143523803e-306;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032508802400e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.26511825540848621482e-98;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580033032602819e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8080dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadac6ab;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xab9babda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadae7e7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -1.09982648853770057025e-175;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9b9bdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf0f4;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = -2.94325462162718820259e-306;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580033032787516e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x808088da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadae7ea;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xea;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -1.50551788040533678720e-267;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadaef;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8888dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaef;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f4;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = -4.36730189107179708777e-306;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580033032972213e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x808888da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadae7ed;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xef;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -4.42431940869604412321e-306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadaf0;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8088dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaf0;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f4;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = -3.37446939240110676561e-267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65377827382188327119e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.68629260532214514337e+163;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580033127660159e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x889bdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaea;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeaeae7e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1dddada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaedef;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -3.00027213925143523803e-306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadaf4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8080dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaf4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = -1.49439718985536910492e-267;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.66665306204672499383e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.54568531029826609031e+192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580033159243330e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8888abda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaedef;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf0efedea;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe7e1dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaeff0;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25620199165454914506e+291;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7c80dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = -3.34465647730970404290e-267;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaeaeff4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x889b9bda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaeaeff4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = -2.41422145261772071466e+163;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe1dad4c6;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdaf0;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.72165207576614628667e+267;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x777cdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = -1.09982648853770057025e-175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9b9bdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xedf4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -2.48409093726987764373e+100;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xd4c6b6da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f4;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.54645768482537526414e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaf4f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d7cdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaf4f5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = -6.70445602260234280209e-267;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x88ababda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xeff4;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.54702413812643084343e-98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xababdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.97490490061958740557e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d80dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = -1.09257744958347109482e-175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9b9babda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xeff0;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.54702413812643084343e-98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaf4f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xababdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaf4f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.07280104673149210486e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaf4f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d777cda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaf4f5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = -2.53416552997451142216e-98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xababb6da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaed;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf0;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.19965297707540114050e-175;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf4f4f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9babdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf4f4f4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.07417995979726760169e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf0f5f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d7780da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf0f5f5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = -2.53988046693092182250e-98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xababc6da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaea;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xed;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.19965297707540114050e-175;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xf0f4f4f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9babdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf0f4f4f4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30115183199464144277e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7cda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = -7.99842402180008500607e-45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xb6c6d4da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadae7ea;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.19965297707540703209e-175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9babdada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaef;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30184128852752919118e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf4f5f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d80da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf4f5f5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = -7.99842402180008127226e-45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xb6c6d4da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadae7e7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe7;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = -4.17538053200258991139e-98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xabb6d4dd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaedf0;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf0f0;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30115183199464144277e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf4f5f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7cda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf4f5f5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -4.56149995569752412743e+100;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xd4d4dadd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae1e1e1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = -4.17967797369005339175e-98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xabb6dae1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1eaeded;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xed;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30022941456294591999e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7780;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -4.65374373328343328688e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xdadadadd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdae1e1dd;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xd4dde1e7;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xd4dde1e7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe7e7;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30022941456294591999e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d7780;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = -4.65577452792301309012e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xdadadddd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdddadad4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.30022648466724875755e+219;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6d6d777c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x80dadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadaf0f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf5f5;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -4.65577449693506028574e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xdadadddd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadad4c6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xab;
	destPtr += sizeof( unsigned char );

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = -2.53362427204529913155e-306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdadadaf0;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x807c777c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8088dada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaf0;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -5.17361930064657957772e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xdadddadd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadad4b6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xabab;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9b;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -9.99561256034801725091e-268;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580122003194730e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x88808080;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x808088da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaf0f4f5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = -1.45681374847422221772e+144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.53014826546691027543e-98;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.08712963198975024839e-175;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032828266478e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xddddddda;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdad4d4c6;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xababab9b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9b9b9b9b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9b9b8888;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x88888888;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaf0;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = -1.74297627385801826954e+144;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.42342205006634515618e-45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.09007173540373037106e-175;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373601829056993200e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xdde1dddd;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdddadad4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb6ababab;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xabab9b9b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9b9b9b9b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeff4f5f5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = -3.21804657286920861676e+163;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdaeff0f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe1e1e1e1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1dddada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdaeff0f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf5f5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = -4.29780235924270334794e+163;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032826912034e+129;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373599743004397980e+129;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe1e7e1e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe1e1e1da;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xedeff0f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -3.40841152022286146443e+192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.65373580032828081781e+129;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xeff0f4f4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe7e7e7e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe7e7e1e7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadada;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdadadaed;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeff0f4f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = -3.41008888809776681930e+192;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.38173075964805545460e+221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe7e7eaea;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeaeaeaed;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xededeff0;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf0f4f4f4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = -3.38172902916813400983e+221;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xededefef;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xeff0f0f0;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf4f4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf4;
	destPtr += sizeof( unsigned char );

}


void drawSprite206( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 1.87078686273839420348e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c414145;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x454b4b4b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4b4b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53064252632970645291e-18;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.52100377080604869497e+93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c3c41;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x454b4b4b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x53535656;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x56565656;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x56;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 2.94790231015135815666e-52;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.02853562081057292374e+137;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x5c5c5c5c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x353c3c3c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x454b4b56;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5c717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717164;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5c5c5c5c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5c56;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16201418297499869247e-62;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962611775634147381e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c3c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x454b5671;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x64645c5c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5c5c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 4.67584111439806631631e-62;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83964627248652169757e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.04218609240700180646e-301;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7164645c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x33333c3c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x454b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717179;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x81818181;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x81818179;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x79797971;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7164645c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5c64;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x64;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16203281809696438724e-62;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.70481363372473338080e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.63135417056761660663e-282;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.40944026954513709018e+277;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c41;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4b717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7181858a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8a8a8a85;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x85858585;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x85858181;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x79797171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x64646464;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x64;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16204820550685512959e-62;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x858a9191;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x858a9191;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9191;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -4.62795217554036394034e-282;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x64646c6c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x85858179;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x79716c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x64646c6c;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16204820550685512959e-62;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x8a9c9c9c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8a9c9c9c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = -1.48407212890283364294e-301;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x81797171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6c716c64;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7171;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16204820550687240193e-62;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x919caeae;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717185;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x919caeae;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = 7.86401675605525235018e+276;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x796c6464;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x716c6c71;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x71;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16204820550685512959e-62;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9caeb8b8;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9caeb8b8;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x565c5c64;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x565c5c64;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7179;
	destPtr += sizeof( unsigned short );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.37301201833493441626e+214;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6c646471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x716c7171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16204820550685512959e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9ca2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.22853777750189909515e+54;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b4b53;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5c647179;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x85;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.80891712378388757009e+175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x645c6471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717179;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16203336868225237991e-62;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9ca2bfbf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c41;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9ca2bfbf;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 5.14293488825634035262e+25;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4545454b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4b717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x818a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 8.24558004328617471757e+136;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5c5c5c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717179;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7979;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.15538647126022052079e-62;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x91aebfc5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x33353541;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x91aebfc5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 2.25908288627450959757e+06;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x41413c45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7185;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.92591859264403006023e+93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x53567171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717979;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16221141057748537613e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaebf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53174116483635294977e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7185;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x91;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25706068381348074626e+54;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717179;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8181;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16221141057748537613e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9cb8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc5;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53174116483635294977e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71718a8a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71718a8a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.59702688627451332286e+06;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x414b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717179;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8181;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x85;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 6.86856493449052121106e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x333c4171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaebf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53064597770533274113e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c3c45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81059488627450959757e+06;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x41457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8181;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x85;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16221141057748537613e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9cb8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc5;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53258819778360725011e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4571;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.32486959966695600388e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71858a8a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71858a8a;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 6.86856493449052121106e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x333c4171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaebf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.89118873067275423028e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c417171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8581;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53385874720448870062e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7171858a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4b71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171858a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 5.16221141057748537613e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x33353c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71ae;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81059488627450959757e+06;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x41457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171718a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8581;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53255676492033023506e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c454b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8a91;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.95002617640891710235e-52;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x353c4171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71b8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.79523488627450959757e+06;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624534425585520e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.53569800983554619215e+213;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x41455371;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x718a8179;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6c5c534b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x454b4b71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8a91;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x91;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.96960208981291267593e-52;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x353c7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9cb8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.24134661910530063388e+54;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -7.72979327823939723986e-224;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.65728862750729407574e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b5c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71858a91;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x919c9c9c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9c918579;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71645c53;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x53537171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8a91;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.54190556020340455384e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9cb8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.08777901106639516198e+55;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.55727615488496578579e-171;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.63443926197670468997e+276;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b5c6471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x818a9191;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9c919c9c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9c918a85;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7971645c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7191;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x91;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53174116483635294977e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71ae;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x565c6c79;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x565c6c79;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8185;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -1.48406765076541129080e-301;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8179716c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x64647171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7191;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53258819778360725011e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4571;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71ae;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x5c646c79;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5c646c79;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 9.66270470333128077558e+276;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439431893774e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7971716c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x64717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717191;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9c9c;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.89118873067275423028e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c417171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71ae;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x5c64;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x64;
	destPtr += sizeof( unsigned char );

	destPtr += 17;

#ifdef powerc
	*( ( double * )destPtr ) = 2.83643426719653017271e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624561359662480e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x71716c6c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71919c9c;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.89118873067275423028e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c417171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71ae;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x5c5c;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x64646464;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x64646464;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x6c;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.37301201841331654923e+214;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6c646471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.32486959966695600388e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
#else
#endif
	*( ( unsigned char * )destPtr ) = 0x5c;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 8.19687424417870098245e+107;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x56565656;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5c5c646c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x71;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.03491967813990049855e+175;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x64646471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9cae;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81059488627450959757e+06;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x41457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71ae;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xae;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 5.22851432554395891811e+54;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b4b4b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x56715c64;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6471;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.24558004328616645437e+136;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5c5c5c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9cae;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81059488627450959757e+06;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x41457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xae;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 2.79113888627450959757e+06;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717981;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x41454b71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717981;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.24558004328616645437e+136;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5c5c5c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xae;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 6.63534343973582312099e+25;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x454b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xae;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53174116483635294977e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717981;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717981;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x85;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20562650787998730919e+107;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x56565c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 6.63534343973582312099e+25;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x454b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53258819778360725011e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717185;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4571;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717185;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8a91;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.92591859264307778609e+93;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7171a2b8;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x53567171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171a2b8;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.23461201866407263292e+54;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b5371;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7191;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53068237365228507435e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c3c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8a91;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25706068381347530175e+54;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71aeb8bf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71aeb8bf;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.23461201866407263292e+54;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b5371;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7191;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53258819778360725011e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4571;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 6.63534343973582312099e+25;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71aeb8bf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x454b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71aeb8bf;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 8.59850194154458953054e+54;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4b567171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9191;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.54190556020340455384e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xae;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.59702688627450959757e+06;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x414b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71b8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.52411324631110040557e+93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x53535c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8a91;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.89118873067275423028e-18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3c417171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x719c;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xae;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81059488627450959757e+06;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x41457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaeb8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.91216859980282000500e+93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x53565671;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x718a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x91;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81059488627450959757e+06;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x41457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71ae;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.32486959966695600388e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaebf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 2.92591859264307778609e+93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x53567171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8a91;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 6.63534343973582312099e+25;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x454b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x71a2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.32486959966695600388e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c457171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb8c9;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20562650787998730919e+107;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x56565c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x858a;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25706068381347530175e+54;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaeb8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb8;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.95165750252591673348e-52;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x353c4571;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaebf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20562650787998730919e+107;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x56565c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8185;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 7.40872268862624808289e+54;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4b535671;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb8bf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.95410449170141618018e-52;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x353c4b71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb8c9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 8.25466552733986154050e+136;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5c5c6471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8585;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x85;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 8.20562650787998730919e+107;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x56565c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfbf;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.89118873067277772662e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c417171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5c9;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 8.25466552733986154050e+136;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5c5c6471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8181;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x81;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.18575632191878389815e+137;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7171b8bf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5c646471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171b8bf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 6.87236316158886226608e-62;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x333c4571;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171719c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfc9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 4.03491967813990049855e+175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x64646471;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8181;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 4.04499162905926422811e+175;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71a2b8bf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x64647179;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71a2b8bf;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.95410449170144251554e-52;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x353c4b71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x717171b8;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9c9;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 4.03490458229020121238e+175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6464646c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7979;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 6.91521262960115945546e+175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x64717981;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x85719c9c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xaeb8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xae;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.95410449170143880633e-52;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x353c4b71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5c9;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62788204512761438823e+175;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x646c7171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7979;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x79;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 4.15219738860465318272e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x71798185;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x919c9c9c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9c;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.95160970976860538607e-52;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x353c4553;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x717171ae;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfc9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 2.34888377625973472592e+214;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6c717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717179;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7171;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6c;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x7981858a;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7981858a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8a;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53171634941798464044e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4153;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171719c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfc9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x716c6464;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x716c6464;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = 1.53256338236523894078e-18;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3c3c4553;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171719c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb8c9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6c64645c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6c64645c;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = 2.26165467533700959757e+06;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9cb8c9c9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4141414b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x56717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9cb8c9c9;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.14707997683257938030e+238;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x6c64645c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x71797979;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6c64645c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5c5c;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.22849666023166620140e+54;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71719cb8;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4b4b4b45;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4b4b5c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71719cb8;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9c9;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.41117819032702075913e+277;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.65984349511311358685e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 8.19687404654879369749e+107;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.91522415376734165037e+93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x79797979;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7164645c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5c5c5656;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x56565656;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x53534b4b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x53565c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7171719c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb8c9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 1.41118497891174585243e+277;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83132689911300799919e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 8.23867229922907417208e+136;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962655722161813765e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x79797981;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x79717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71716464;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5c5c5c5c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5c5c565c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x565c5c71;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x91aebfc9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.93950788464147259577e+277;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x71919cb8;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x79818185;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x85817171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71919cb8;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfc9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = -4.63135417056760437416e-282;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439430043028e+238;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962655653415456620e+238;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x85858585;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x85858171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x919c9ca2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfbf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -6.90484436814270660920e-258;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.83962624439431893774e+238;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9c9caea2;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8a8a8a8a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8a8a8585;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717171;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x71717191;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9c9caea2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfbf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = -4.74636429408683749674e-224;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -7.40367136035717453454e-171;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x91919191;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x91919c9c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9c9c9c9c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xaea2b8bf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = -7.40367108345500193682e-171;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9c9c9c9c;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9caeaea2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb8b8;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

}


void drawSprite207( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 5.08131805557700788278e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x151a1a1b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1b232323;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2328;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10795619121236719753e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.01083485205056541653e-139;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a1a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1a1b2323;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x23232828;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x28282828;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x2f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10795629600505381053e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.28350187976558790224e-114;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x2f2f2f2f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a1a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x23232328;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x28494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949492f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2f2f2f2f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x2f2f;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x2f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10798303033274069918e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780548168661058104e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a23;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x23282849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x37373737;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3737;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x37;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10798308867257272868e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494040;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a23;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x282f4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494040;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4040;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x40;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10875845997881145955e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780554307067706160e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151b28;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2f494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4e4e4949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x49;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 4.11484225467999057887e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.56115901089137156237e+180;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.69815863393535786259e+262;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972696170527e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15152328;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49495560;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x656b7676;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x76767676;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x766b6b6b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6b494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949554e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4949;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x49;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10875876166980125762e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.28551672042095413762e+286;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.88490777791317664334e-292;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.53865753746588510935e+286;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x554e4940;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15151b28;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949606b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7b838383;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x83838383;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x837b7b7b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7b7b7b7b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7b7b7b76;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x766b655b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x554e4940;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x49;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10799832504417606619e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a28;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49496576;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8393;
	destPtr += sizeof( unsigned short );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 6.53870491851965047388e+286;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7b766560;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7b7b7b83;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x83838383;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7b766560;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x55;
	destPtr += sizeof( unsigned char );

	destPtr += 1;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x40404044;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x40404044;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10799832504329080144e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a28;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49496076;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x89;
	destPtr += sizeof( unsigned char );

	destPtr += 27;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x40374040;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x40374040;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x49;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10418128962875897528e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151523;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949496b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8993;
	destPtr += sizeof( unsigned short );

	destPtr += 27;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2f373740;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2f373740;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x494e;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10418128962873546044e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151523;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7b93;
	destPtr += sizeof( unsigned short );

	destPtr += 28;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x282f4940;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x282f4940;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x494e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x55;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10415752598415677094e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1515151b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7683;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53572999355530470046e-139;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2328284e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494e55;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10809635007307455520e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7b89;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x89;
	destPtr += sizeof( unsigned char );

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 5.90946845854771855771e-178;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1b232849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494e5b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x5b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10429416694048423555e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15151549;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x497b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x83;
	destPtr += sizeof( unsigned char );

	destPtr += 27;

#ifdef powerc
	*( ( double * )destPtr ) = 5.08829916165486725956e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x151a2349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494955;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5b5b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x60;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 4.11494027971173713056e-207;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x15152349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x497b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x83;
	destPtr += sizeof( unsigned char );

	destPtr += 26;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10798347276132578008e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x495b6060;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a23;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x495b6060;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10809635007307455520e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49497676;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49497676;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.02282016418893984881e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.00878476312469558711e-139;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1b1a151a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1a1b1b23;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x23232328;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2f373737;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x37;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.08829916165485896020e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49496065;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x151a2349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49496065;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 4.11494027971173713056e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49496b6b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15152349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49496b6b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.36889476716787261063e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.01084610788035866493e-139;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x231a151a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x1a1b2323;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2323282f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2f374040;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4037;
	destPtr += sizeof( unsigned short );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.08221566864271444877e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494960;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x151a1b49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494960;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x656b;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 4.10809635007307455520e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x4949496b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15151a49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949496b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 4.02594360931138002706e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1b1a1a49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x49;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 4.02645665865766169575e-178;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1b1a1b23;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4965;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 4.11494027971173713056e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494960;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x15152349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494960;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x60;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 2.00514754166825436354e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972720715412e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x23231a49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49495b5b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x55;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.00883778122929585516e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553001862909363e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x23232349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49656b76;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 5.11719575346254538952e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x4949495b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x151a4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949495b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x55;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 2.00883778122929585516e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552996645925009e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x23232349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49606560;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x5b;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 2.42853586114545962609e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972761533761e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x28232349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949656b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 5.31186752985116975545e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494955;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x151b4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494955;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x4e;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 2.44737214821928671017e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780562058266820643e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x28234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6b766b65;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 3.06295742270777691815e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x28282328;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b76;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.08829916165485896020e-207;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x151a2349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4e49;
	destPtr += sizeof( unsigned short );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 3.08185760754819901599e-115;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x28284949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x767b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x6b;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.06549977929987549560e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x28282849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b76;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 6.15137623444803087818e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972610619957e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.05472129256244429647e-81;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1a1a2349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494037;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2f2f2f49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7b7b;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.06549977929987549560e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x4976767b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x28282849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4976767b;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 6.42165380720966110073e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552963264485612e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.99886074028161026499e-43;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x4949497b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1a1b4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49404037;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x372f3749;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949497b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8383;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x76;
	destPtr += sizeof( unsigned char );

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.59166530977251922713e-81;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49496b7b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2f282828;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49496b7b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49497b83;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49497b83;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x837b;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.06549977929987549560e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x4949497b;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x28282849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949497b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x83;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49498389;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49498389;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x83;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.06302132047437193191e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x28282349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7b83;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.94923213632049366139e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49838989;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1b234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49838989;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 2.43101431997096318978e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x28232849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8383;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.02441879270924881979e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x83899389;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x23234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x83899389;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.42853586114545962609e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x28232349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4983;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x89;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.02441879270924881979e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972648205798e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x23234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949497b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8993;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 2.00883778122929585516e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x23232349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4983;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x89;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.54925286361292762853e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972648427637e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x23284949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494989;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9393;
	destPtr += sizeof( unsigned short );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 2.00883778122929585516e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972908200936e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x23232349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49498993;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53572198522244215924e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972883862044e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x23282849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49498393;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.42428614733759097457e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553033138844495e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x231b2349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49838993;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.06896962165558048477e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972948765755e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x28282f49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49499393;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 5.94923213632049366139e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553039410165544e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1b234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49899393;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.97013725060867624413e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x282f4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8993;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.37180274024722309369e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972908200936e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x231a2349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49498993;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.95724926471605771292e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494983;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x282f2f49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494983;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9393;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 5.90344365888353886315e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553033179409314e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1b232349;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49839393;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.95724926471605771292e-115;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494989;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x282f2f49;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494989;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x93a5;
	destPtr += sizeof( unsigned short );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553039410165544e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49899393;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 2.06141311742875273406e-81;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x498393a5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2f2f4949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x498393a5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.94923213632049366139e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1b234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4993;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 2.05678031559823150804e-81;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x498393a5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2f2f3749;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x498393a5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.94923213632049366139e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1b234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8993;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 2.05678031559823150804e-81;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2f2f3749;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4989;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.94923213632049366139e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1b234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9393;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 3.05934083687341069364e-81;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972648205798e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2f373749;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x4949497b;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9393;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.94923213632049366139e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494989;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1b234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494989;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x93a5;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.04103974118916888284e-42;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972648332563e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x37373749;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494983;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8993;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.94923213632049366139e-178;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49498393;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1b234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49498393;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa5a5;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 1.45745069190555055632e-42;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972851251733e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x37404049;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49497b89;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x494993a5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x494993a5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.25020839317172161032e+01;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972851156659e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x40404044;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49497b83;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x498993a5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x498993a5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 3.25725490196078411032e+01;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x40404949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4976;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7b;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x8993a5a5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8993a5a5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6b76;
	destPtr += sizeof( unsigned short );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972648427637e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494989;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x93a5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49494965;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494965;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x65;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972908200936e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49498993;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa5a5;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.12868005540946094701e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49496060;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x49494e4e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49496060;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x5b;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.07780782519606576749e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553039410450766e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1a234949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x498993a5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.63408992774007404462e+69;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x495b5b55;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4e4e4e4e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x495b5b55;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.28677312117607364578e-207;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553039410450766e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x151b2849;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x498993a5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.63557005908938446946e+69;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49554e49;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4e4e5555;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49554e49;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x40;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 6.38662660370311160574e-183;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780553039410450766e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x1a1b232f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x498993a5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19584802583307213082e+103;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.44310791587386181737e+131;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.01084627099038354201e-139;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780570063913390530e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x55555b5b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5b494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5b554940;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x372f2823;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2323282f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x899393a5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.21449457853202509369e+132;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.35080680850590969962e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.06860644053445314569e-81;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x49839393;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5b5b6060;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x65604949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x494e4940;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x372f2f2f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2f374949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49839393;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa5a5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 2.29499608036927647636e+156;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972647413516e+45;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780566331649558574e+45;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x60656565;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x65606049;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7b898993;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x93a5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 3.55556525462151642354e+180;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.12780552972648126570e+45;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x7b899393;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x656b6b6b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x65656060;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494949;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x49494976;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7b899393;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x93a5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 2.81700905511843026678e+209;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 4.61956548833336434845e+209;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6b6b6b6b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6b6b6b6b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6b767b83;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x83939393;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 4.42464057612168138228e+262;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x76767b7b;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7b838389;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8993;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x93;
	destPtr += sizeof( unsigned char );

}


void drawSprite208( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 1.85690464060124939206e+161;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x616a6a6a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x707a7a7a;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7a7a;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22177356108596538123e+161;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.73723499431940474092e-287;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6161616a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x707a7a7a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x84848484;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x84848484;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x84;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 8.35971393995764876729e+112;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.32548204395671829275e-208;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x94948686;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5761616a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x707a8486;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x94c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c594;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x94948686;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8686;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x86;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62280153283555879588e+112;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729524171919178e+28;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524341328018547689e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5757616a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x707a86c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa9949494;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9494;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x94;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 5.61337433613904778579e+112;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.49314992024078378553e+47;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.12995094948001219049e+105;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5949494;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x57575761;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x707ac9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5d5d5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdfdfdfdf;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd5d5d5d5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5949494;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9494;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 5.61340751006230293379e+112;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -7.28156236278834818919e+47;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.87958128594975412561e+168;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -9.83314601631407281443e+27;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5757576a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7ac5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9dfe2e9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe9e9e2e2;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe2dfdfdf;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdfd5d5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5bfc5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa9a9a9a9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62280275547087451204e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xe2e9f2f2;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5757616a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe2e9f2f2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf2f2;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -4.57439002694851274211e+153;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5a9a9bf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xdfd5d5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5bfc5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5a9a9bf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62280275569397408597e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xe9f2f9f9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5757616a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe9f2f9f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf9;
	destPtr += sizeof( unsigned char );

	destPtr += 13;

#ifdef powerc
	*( ( double * )destPtr ) = -1.84545538411507870519e+105;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xd5c9bfa9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5bf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfbf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62280275569397750355e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xe9f9fafb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5757616a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe9f9fafb;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -1.80762330470053590743e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc9bfa9a9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5bf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5c5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.61340858536288217064e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xe9f9fbfd;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5757576a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe9f9fbfd;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x869494a9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x869494a9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xc5;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -5.01233872227078089412e-02;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xbfa9a9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5bf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9c9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 5.61340858536288217064e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5f9fbfd;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5757576a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5f9fbfd;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -4.34726795960997042063e-287;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x847a7a7a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x86a9bfc9;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -5.46363826789115080437e-108;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa9a9a9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9c9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 5.61340858536288217064e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5f2fafd;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5757576a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5f2fafd;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 3.28380846809420597789e+233;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x706a707a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9a9bfcf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xd5;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -1.56506547783154409830e-209;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9cfd5d5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x949494c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9cfd5d5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.61340858536288217064e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9f9fb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5757576a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9f9fb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfd;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 4.79524479834645633125e+281;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7a6a6ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd5df;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xdf;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -3.21383465453955611978e-277;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8686c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9d5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd5;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62313674620682806621e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x575761c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 5.96860807354261618073e+281;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7070c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdfe2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe2;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 1.50826281175074313258e+282;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9d5d5d5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a84c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9d5d5d5;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62313674620682806621e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x575761c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfb;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 9.61317967403306159943e+281;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7a7ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe9;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 9.72521193089223573586e+281;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5d5d5df;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7ac9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5d5d5df;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xdf;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 5.63159144239045145758e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5f9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x57576ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfa;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = -6.75808463315632604213e-287;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x848494c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 6.65039383833264309345e+233;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5dfdf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x707ac5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5dfdf;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe2;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62313674620595658350e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x575761c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf2f9;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -3.91611350010327143292e-209;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x94a9bfc5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf8f2;
	destPtr += sizeof( unsigned short );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 5.26357162810052233033e+204;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5df;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6a70c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5df;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe2e9;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 8.37729375906840980958e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x57616ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf2f8;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -1.35353815263704823137e-107;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa9bfc9d5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe2c5c9f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf9e9;
	destPtr += sizeof( unsigned short );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25869180659263020890e+204;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9df;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6a70c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9df;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe2e9;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 5.62313674620595658350e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x575761c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -4.99529008839213660742e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc9d5dfe9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf2f9f9f2;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.15105196353778597130e+204;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6a6a7ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9e2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 8.37729375906840980958e+112;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x57616ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5e9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe9;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xe2f2f2f9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe2f2f2f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf2f2;
	destPtr += sizeof( unsigned short );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 3.28380843940832138005e+233;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729518610875237e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x706a707a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5e9f2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 8.54829701543043039363e+112;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524353246785417764e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5761c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9e2e2e2;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 12;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0xf2f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.09357550275884361806e+233;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776971504724078553e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x70707ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5e9e9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.88199048706284905825e+161;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.34862265046567112562e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x616ac5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5dfdfe2;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = -7.78627634138937652715e-210;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729524171919178e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x94847a7a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22434287122259138223e+161;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524353204828691765e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x61616ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5df;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdfe2;
	destPtr += sizeof( unsigned short );

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -2.18491782545126647977e-108;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729524171919178e+28;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9e9f2;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xa9948686;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x86c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9e9f2;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22599469646501347524e+161;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.34862265014715338227e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x616170c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5df;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe2e2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe2;
	destPtr += sizeof( unsigned char );

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = -4.99617058857313792175e-02;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776971504390311221e+47;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5f2f2;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xbfa99494;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa9a9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f2f2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf8;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.88199048706310379293e+161;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48599259718830584950e+47;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xe9e9e9e2;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x616ac5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe9e9e9e2;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -5.46354718115783502734e-108;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729524171919178e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa9a9a9a9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xbfc5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe9f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.88199476086709453961e+161;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524353204828207980e+28;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5f2f2e9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x616ac5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5f2f2e9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xdf;
	destPtr += sizeof( unsigned char );

	destPtr += 17;

#ifdef powerc
	*( ( double * )destPtr ) = -9.82584575626704102544e+27;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776971504392136638e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc5bfbfbf;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.88199048706310379293e+161;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776968792723661445e+47;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xf2f8f2e9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x616ac5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf2f8f2e9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xdf;
	destPtr += sizeof( unsigned char );

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = -9.82584575626704102544e+27;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776971504890881090e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc5bfbfbf;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9f9;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.25869180659263020890e+204;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729518661694665e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6a70c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5f2f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf9f9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr += 21;

#ifdef powerc
	*( ( double * )destPtr ) = -5.01225527576881005332e-02;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xbfa9a9a9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9f9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf9;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.26357162810052233033e+204;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776971504890921655e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6a70c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fa;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfaf9;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x7070;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7a;
	destPtr += sizeof( unsigned char );

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = -5.46363826789114777891e-108;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xa9a9a9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf9fa;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 8.39397712541332936854e+204;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729593646672441e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6a7ac5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5f9fafb;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x706a6a70;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x706a6a70;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7a84;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x94;
	destPtr += sizeof( unsigned char );

	destPtr += 13;

#ifdef powerc
	*( ( double * )destPtr ) = -1.57962072819566888037e-209;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9494c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfb;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 6.65037873601657364563e+233;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48777004312917123370e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x707ac5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf9fafbfb;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.79499189000830890098e+281;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7a6a6a6a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7a7a8694;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x94;
	destPtr += sizeof( unsigned char );

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = -5.80515049660131552129e-277;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5fa;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x869494c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5fa;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfb;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 9.62736088615084939071e+281;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7a84c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf9fb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfb;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 4.79499270464953707892e+281;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x94a9a9bf;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a6a6a6a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c994;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x94a9a9bf;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = -6.74014732602962382437e-287;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x848486c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfb;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 9.62736097236983244926e+281;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f2;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7a84c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfafb;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 2.73037584067666987663e+204;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa9a9bfc5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6a616ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa9a9bfc5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.50826281175074313258e+282;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9f9fbfd;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a84c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9f9fbfd;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.50825839740616808795e+282;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a84c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfb;
	destPtr += sizeof( unsigned char );

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 3.28103963475848541247e+233;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x706a6ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9d5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd5;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.72521193089223573586e+281;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7ac9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5fb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfd;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = -6.75808463315632604213e-287;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9f9fb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x848494c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9f9fb;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 3.28395057668483816859e+233;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x706a70c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd5df;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe2;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 6.65039383833264309345e+233;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x707ac5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf9fb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfd;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = -6.74014732602962382437e-287;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fa;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x848486c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fa;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 4.19698856256123466183e+204;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776968859560732025e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6a6ac5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5e2e9f2;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 5.26357162810052233033e+204;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6a70c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfafd;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfd;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = -7.47768128251292580667e-287;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8486c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf9;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 4.17023108356529096728e+233;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729583513375367e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7070c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5f2f2f2;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.35807896093944020959e+161;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6170c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfd;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = -7.41481059934608098236e-287;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x848694c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc5f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf8;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 9.71956148361850595992e+281;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776971504890881090e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7ac5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9f9;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.36026714858272648428e+161;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6170c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfdfe;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = -3.18463686952953537774e-277;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x868694c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9f2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf2;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 1.49038996150183681717e+282;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729593646650451e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7a8486c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5f9fafa;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 8.38856687769445586072e+112;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x576170c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfe;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = -1.57130343453345009147e-209;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9494a9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe2e9;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -3.18463686952953738191e-277;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.34862266561375892992e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x868694c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9fafbfa;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 1.27361887926967331376e+113;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x576a7ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfdfe;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = -1.57130345259312755615e-209;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9494a9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdfe2;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe2;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -3.90305277825862982423e-209;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9fa;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x94a9a9c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9fa;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfb;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.88199476080496375178e+161;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x616ac5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5f9fb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfefe;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = -1.05225845454315951727e-141;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa2a9a9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9d5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdfdf;
	destPtr += sizeof( unsigned short );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -5.48194713258205311135e-108;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xa9a9bfc9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f9fb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbf9;
	destPtr += sizeof( unsigned short );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 8.40735510412800714953e+112;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x57617ac5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5fb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfdfe;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = -5.46361872593650566153e-108;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa9a9a9bf;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd5d5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd5;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -1.85548433727215439358e-107;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xf9f9faf9;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xa9c5c9d5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe2c5c5f2;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf9f9faf9;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 8.38810450837181131878e+112;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x57617086;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c5f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfdfe;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -1.35186441810396111214e-107;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa9bfbfc5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5d5;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9c9;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc9;
	destPtr += sizeof( unsigned char );

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -3.41178873506393198767e-01;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xbfd5d5df;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe9f2f2f8;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf9f2;
	destPtr += sizeof( unsigned short );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 8.37681671091017358391e+112;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x57616a84;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfe;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -1.69915410591370524829e-01;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xbfc5bfc9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc9c5;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xbf;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xd5d5e2e2;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xd5d5e2e2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe9e9;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22426240768047513103e+161;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x61616a7a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5f2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfe;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = -1.70220586365344567215e-01;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xbfc5c9c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c5c9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xbfbf;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa9;
	destPtr += sizeof( unsigned char );

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.22592068536525043144e+161;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x61617084;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5f2;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfe;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.94449591857601831721e+47;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xbfa9a994;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xc9c9c9c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xbfa9a994;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = 4.14107418695395635321e+204;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xf2fbfdfe;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6a6a6a7a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x84c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf2fbfdfe;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -2.94449591857601831721e+47;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xbfa9a994;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xc9c9c9c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xbfa9a994;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9494;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.61276249132001300575e+281;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc9c5f2fb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7a7a7a7a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7a8494c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5f2fb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfdfe;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = -4.98629948541276425456e+47;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.01219207165287949923e-02;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.56083344313288439290e-209;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -7.41481052136412223784e-287;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc9d5d5d5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xbfa9a994;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x94949494;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x94948686;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8484847a;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x848694c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5f9;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfd;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = -3.12995105187441016912e+105;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.34085889810463831894e+28;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.56500833496258882987e-209;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.34862281686264969514e+28;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xd5d5d5d5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5a9a9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x94949494;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x94949494;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x869494c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf2f9fbfe;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = -3.13557237254380923577e+105;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729518407223694e+28;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.34765928738464313680e+28;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xc5e9f9fa;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xd5d5dfdf;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdfdfc5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c5c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c5c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5e9f9fa;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfdfe;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = -2.22737782319431546563e+168;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776971504390311221e+47;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.48776993402592924215e+47;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe2e2e2e2;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe2dfdfc5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc9c5c9c5;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe9f2f9fa;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfd;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfe;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -1.58685394086110418314e+202;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.59524729524172622865e+28;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xf2f9f9fa;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xe9e9e9e9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe2e2e2e2;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5c9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc5c9c5e9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf2f9f9fa;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfbfd;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfd;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = -2.32072589775453671015e+202;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.18182227303052221638e+245;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xe9f2f2f2;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf2f2f2f2;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf2f2f8f9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf9fafbfb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfd;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = -5.18288915527594247878e+245;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xf2f2f9f9;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf9f9fafa;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfafb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfb;
	destPtr += sizeof( unsigned char );

}


void drawSprite209( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 1.73694617843266333426e+98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5454545d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d5d5d5d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6666;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x66;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 1.43675849826735154728e+74;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.90359856653338950793e+185;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4f545454;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d5d6666;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x66666666;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x66747474;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x74;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 9.15759747556578097921e+49;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.02231794676030740789e-166;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x74747474;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4f5454;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d5d6674;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d7f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x74747474;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7474;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69609583885115073669e+49;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223927713078879738e-137;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223909199625416129e-137;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a5454;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d66749d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8c7f7f7f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7f7f;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x7f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69038684904402342486e+49;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300625557977020606e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300625557977020606e-166;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9d8c7f7f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a4f54;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d749da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d8c7f7f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7f8c;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69613614115697354573e+49;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.70351788224600123936e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.38043765966949029001e+86;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300591008838664520e-166;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a545d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x66a39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3c3c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1d1d1d1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1d1c3c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc3b7b79d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8c8c8c8c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8c;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69613709939211879510e+49;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.13620977380567040000e+18;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -9.85201316816951849999e+148;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.71363832571356018964e-40;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a545d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc3d1d1de;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe4e4e4e4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdeded1d1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1c3c3c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb7b7a39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9d8c8c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8c9d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69617724073350938092e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xd1dee4eb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a5466;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dc3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1dee4eb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xebeb;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -2.84836781336502118400e+18;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39d9d9d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xc3c3c3b7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb7aca39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9d9d;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69617724073350938092e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xd1e4ebf1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a5466;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dc3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1e4ebf1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = -2.71768461608537229113e-40;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9d9d9d9d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xb7b7aca3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9d9d9d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69617724073352391935e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a5466;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dd1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdeeb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = -1.71621259008980411651e-93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xacaca39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9da3;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69613709939215202580e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a545d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dc3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdeeb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr += 21;

#ifdef powerc
	*( ( double * )destPtr ) = -5.27103000665783359045e-137;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa3a39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3a3;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xac;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69042810862129495557e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a4f5d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdeeb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = -5.02231794676032909717e-166;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x9d9d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3ac;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xac;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 7.68471911785047111605e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a4a5d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd1e4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = -3.18998577400048543829e-248;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8c8c8c9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dac;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xacb7;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69642265305865310772e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a549d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdef1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr += 26;

#ifdef powerc
	*( ( double * )destPtr ) = -3.18998577400048543829e-248;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8c8c8c9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dac;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb7b7;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69642265305865310772e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a549d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd1e4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr += 12;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x54545d66;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x54545d66;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x74;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38860926770096176192e+306;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7f7fa39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3acb7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb7b7;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 9.16820036964904581642e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4f5d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdeeb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 7.68467785826520033280e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a4a54;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d747f8c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38244139621893484426e+306;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7f7f7fa3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9d9db7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb7c3;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69642265305706426488e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a549d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9dde;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xeb;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 7.68467786146385458185e+49;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a4a54;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5da37f8c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9dac;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb7;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 9.39287275389423042518e+252;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x74747f9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb7c3;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xc3;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19057935350421063312e+50;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3de;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xeb;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 7.68467908063551180798e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9dacb7c3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a4a54;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9dacb7c3;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.45731028771084141415e+252;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x7474a39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da3b7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc3c3;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 1.20519972197501855631e+50;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a549da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9dde;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xeb;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.73703776496765887501e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9d9dc3d1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x545454a3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9dc3d1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 3.48401007858947160809e+185;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x66747f9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc3c3;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19057935350421063312e+50;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39ddee4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39ddee4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53034890824615674843e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39dd1de;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x545d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39dd1de;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 3.47672249163511104479e+185;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x667474a3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc3d1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19057937432981050340e+50;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da3d1e4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3d1e4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 8.61825825815785744701e+141;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da3e4e4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5d669da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3e4e4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 3.50394331376699330429e+185;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x66749da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da3c3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc3d1;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 1.76140136709613175563e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9d9dd1de;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x54549da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9dd1de;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.39287275389423042518e+252;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa3e4ebf1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x74747f9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3e4ebf1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 1.92391983210771786415e+185;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6666a39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd1d1;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.43932212075133467157e+74;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da39dd1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4f545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dd1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xd1;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38467013418319137609e+306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9df1f1f1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7f7f8ca3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9df1f1f1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 8.61825825815785744701e+141;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5d669da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd1d1;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.74003370495542001794e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39da3c3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x54545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da3c3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xd1;
	destPtr += sizeof( unsigned char );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = -8.79348815772861585417e-248;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8ca3acb7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3e4f1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf1f6;
	destPtr += sizeof( unsigned short );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 8.61822350565107086125e+141;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5d669d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd1d1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xd1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.76140136709613175563e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da39dc3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x54549da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39dc3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc3d1;
	destPtr += sizeof( unsigned short );

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = -2.84824753402129986031e-93;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xacb7c3d1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdeebebf1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 5.64733225586385084669e+141;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5d5da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xd1d1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xde;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53034890824615674843e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39da3b7;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x545d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da3b7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xc3c3;
	destPtr += sizeof( unsigned short );

	destPtr += 11;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xb7c3d1de;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xb7c3d1de;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe4eb;
	destPtr += sizeof( unsigned short );

	destPtr += 13;

#ifdef powerc
	*( ( double * )destPtr ) = 5.64288400260602113149e+141;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5d5d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9dd1;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xde;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53035670002765938529e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x545d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb7b7;
	destPtr += sizeof( unsigned short );

	destPtr += 30;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53235139609186818235e+98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x545da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9dde;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xde;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.64286662661776843964e+141;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5d5d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb7b7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb7;
	destPtr += sizeof( unsigned char );

	destPtr += 29;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53035670002765756404e+98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x545d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3de;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe4;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 5.64733225586385084669e+141;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300655953321828051e-166;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5d5da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xacacacac;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 28;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53034890824615674843e+98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x545d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9dde;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe4;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 8.61825825762757624495e+141;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223922955113537952e-137;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5d669da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9dacaca3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr += 17;

#ifdef powerc
#else
#endif
	*( ( unsigned char * )destPtr ) = 0xa3;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53035670002765938529e+98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x545d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9de4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe4;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 8.61822350564899948203e+141;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300625511058772736e-166;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5d669d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9daca3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa39d;
	destPtr += sizeof( unsigned short );

	destPtr += 15;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x9da3;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.53034890824615674843e+98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x545d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3e4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xeb;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.90831762329548419032e+185;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223927713006277266e-137;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x6666749d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9dac;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3a3;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

	destPtr += 13;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x8c9d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xac;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.43932214592793855721e+74;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa3deebeb;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4f545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3deebeb;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 1.90831765356918907513e+185;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300625557976297630e-166;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xb7aca39d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6666749d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb7aca39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8c7f;
	destPtr += sizeof( unsigned short );

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x66748c9d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x66748c9d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb7b7;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19058470568305952662e+50;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9de4ebf1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9de4ebf1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 3.50392781386750652091e+185;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300625557976297630e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.71363820859048937889e-40;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.90359863792602573834e+185;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x66749d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb7b7a39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8c7f7466;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x66666666;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x748c9db7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xc3;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.20656454848811725407e+50;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a54a39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3e4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 9.39287259065874637047e+252;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.02629260954962492710e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -7.66090609367924220581e+85;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 9.39266283403276892334e+252;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x74747f9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39dd1d1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1c3b7ac;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d8c7f7f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x74747f7f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9dacb7c3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xd1;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.76140136709613175563e+98;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x54549da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdeeb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 9.37322529509656801545e+252;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.97916854777998919440e-137;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.38042332424515491045e+86;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.97910252664829819248e-137;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x747474a3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9dc3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1d1dede;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1d1c3b7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xaca39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9dc3d1de;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19057935350421187927e+50;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9de4f1f1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9de4f1f1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 9.44661248638766010354e+252;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39dc3d1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x74749da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39dc3d1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xdee4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe4;
	destPtr += sizeof( unsigned char );

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = -2.71770044227361036345e-40;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xd1dee4e4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xb7b7acac;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xac9da3c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1dee4e4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19058470560170952713e+50;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xa3eb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 9.39287275389173968737e+252;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da3c3d1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x74747f9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3c3d1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -2.72267345896592292003e-40;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xb7b7b7c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc3d1d1e4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xebeb;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 7.70669873231813666746e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa3e4f1f6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a5d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3e4f1f6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( double * )destPtr ) = 9.39291454217814843143e+252;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39dc3d1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x74747fa3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39dc3d1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 10;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0xc3d1d1de;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xc3d1d1de;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe4e4;
	destPtr += sizeof( unsigned short );

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19058470560170952713e+50;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9debf6f6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9debf6f6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38466611602591349204e+306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa3c3c3d1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7f7f8c9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3c3c3d1;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 25;

#ifdef powerc
	*( ( double * )destPtr ) = 7.70669873231813666746e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa3e4f1f6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a5d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3e4f1f6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( double * )destPtr ) = 1.38244139621917029104e+306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9db7c3c3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7f7f7fa3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9db7c3c3;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 24;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69617724073347615022e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9ddef1f6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a5466;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9ddef1f6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = 2.50600631294213839734e+306;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9db7b7b7;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7f8c8ca3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9db7b7b7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 23;

#ifdef powerc
	*( ( double * )destPtr ) = 1.19057937432981050340e+50;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa3ebf6f6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a545d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3ebf6f6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( double * )destPtr ) = -3.18998577400048186559e-248;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9dacacac;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8c8c8c9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9dacacac;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xac;
	destPtr += sizeof( unsigned char );

	destPtr += 20;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69613709939211879510e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da3e4f1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a545d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3e4f1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( double * )destPtr ) = -6.61828374107331073112e-248;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da3a3a3;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8c9d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3a3a3;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9d;
	destPtr += sizeof( unsigned char );

	destPtr += 18;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69613614115697354573e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da3e4f1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a545d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x66a39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da3e4f1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf6f6;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = -5.02231794676032909717e-166;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9da39d97;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9d9d9d9d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39d97;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8c;
	destPtr += sizeof( unsigned char );

	destPtr += 15;

#ifdef powerc
	*( ( double * )destPtr ) = 7.69038685183592767645e+49;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39da3e4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4a4a4f54;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da3e4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf1f6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = -5.02630807492137247928e-166;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9d9d8c8c;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9d9da3a3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9d8c8c;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7f7f;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x74;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.30400877950876297661e+73;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223927936069043054e-137;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x4f4a5454;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x545d669d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3e4ebf6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = -6.66095546032083717742e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.01097863932223186769e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.59501938663106925225e+141;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.73996197073351339902e+98;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa39da3e4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9da3a3a3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9d8c7f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7f746666;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d5d5d54;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x54545454;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x54545d66;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da3e4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xebf6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = -5.27738951950136748902e-137;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.02621461507810285100e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 1.24773687536061370670e+185;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.02233347231485602552e-166;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xa3e4f1f6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xa3a3a3ac;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da37f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7f747466;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x665d5d5d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5d5d5d66;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa3e4f1f6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = -7.70527193150178565822e-137;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.02629260954249879406e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223927713005847666e-137;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300625512609797161e-166;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xa3acacac;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9ddeeb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf1f6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = -2.84258169333189243094e-93;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.02629260954249879406e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223927713005847666e-137;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300769661848321721e-166;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xacb7b7b7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb79d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe4ebf1f6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = -2.72265235668125176675e-40;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -5.02629260954249879406e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223927713005847666e-137;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x9de4ebf1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xb7b7b7b7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb7b79da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9d9da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39d9da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9de4ebf1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf6f6;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = -2.84839430549924915200e+18;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300625557977020606e-166;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -6.65300757476861023641e-166;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xc3c3c3c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc3c3c39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9da39da3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdee4ebf1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf1f6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -2.84839430549926860800e+18;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -3.98223927713081982408e-137;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xdee4ebf1;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0xc3c3c3c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xc3c3c3c3;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da39d;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xa39da3de;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdee4ebf1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf1f6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = -1.38470391314097793491e+86;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -9.87583163244723244429e+148;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xd1d1d1d1;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xd1d1d1de;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xdedee4e4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xebebf1f1;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = -1.38866546191832269658e+86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xd1d1dede;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe4e4e4eb;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xebeb;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf1;
	destPtr += sizeof( unsigned char );

}


void drawSprite210( Rect *inDestRect )
{
	unsigned char *destPtr;
	unsigned char *rowStart;

	rowStart = sBaseAddr + inDestRect->top * sRowBytes + inDestRect->left;

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09245332861127830113e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29292929;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2e2e2e2e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x3838;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x38;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09245330382669004609e-110;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.11753456186481594844e-38;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29292929;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x292e2e2e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3838383f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3f3f3f48;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x3f;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09245330382669004609e-110;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54933129759365440369e+194;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x52524852;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x29292929;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x292e2e2e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e48;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x52524852;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x4852;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x48;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09245332861129543753e-110;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.08585630132667045507e+146;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.08581834231965298932e+146;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29292929;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2e2e2e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5252;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x52;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 3.03428339415275619676e-86;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54933129759532847388e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54933129759367515127e+194;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x5e68685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e2e2e2e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3838685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e5e;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 7.12557032755680479430e-38;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54933129759532847388e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.60381768859444519792e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -2.29814077935810791564e-156;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x38383f3f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x485e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685eb4cc;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xccccccb4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9f9f8d73;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x73685e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x5e;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 9.19310230809555500414e-38;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.66182889507269855182e+88;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.75147099887893860024e+196;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -1.56125320177849323740e+187;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x383f4852;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x52525252;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5252685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x68cce6f7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf7f7f7f7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe6ccb49f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8d7e7368;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e5e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5e;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 7.43671510373699251248e-04;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 2.28783624055328155920e+146;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x68e6f7fc;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f485e5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x6873685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e52525e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x68e6f7fc;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xffff;
	destPtr += sizeof( unsigned short );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x8d737368;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x8d737368;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e5e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x5e;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f525e73;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f525e73;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 6;

#ifdef powerc
	*( ( double * )destPtr ) = 3.66182889507279466830e+88;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5252685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e68e6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfcff;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr += 12;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x7368685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7368685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e68;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 3.65408845562021387280e+88;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x52525e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68e6fc;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xffff;
	destPtr += sizeof( unsigned short );

	destPtr += 14;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x685e5e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x685e5e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e68;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x4852687e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4852687e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9f;
	destPtr += sizeof( unsigned char );

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 2.50020809507810868575e+40;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x48525e5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x68ccf7ff;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr += 16;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x5e5e5e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5e5e5e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e68;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xb4;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 7.44863704134956952467e-04;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3f48685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x9fe6ffff;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 18;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x5e5e5e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5e5e5e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6873;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f525e73;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f525e73;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9f;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f485e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f485e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4f7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr += 20;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x52525e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x52525e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x68;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x4852;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x68;
	destPtr += sizeof( unsigned char );

	destPtr += 11;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3848685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3848685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xccfc;
	destPtr += sizeof( unsigned short );

	destPtr += 22;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x52525e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x52525e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x73;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x3f48;
	destPtr += sizeof( unsigned short );

	destPtr += 13;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x38485e8d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x38485e8d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe6fc;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e383f48;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e383f48;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x48525e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x48525e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x737e;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x3848;
	destPtr += sizeof( unsigned short );

	destPtr += 13;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3848688d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3848688d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe6fc;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x292e3f52;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x292e3f52;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x5e;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x4848525e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4848525e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x737e;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x2e38;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x48;
	destPtr += sizeof( unsigned char );

	destPtr += 13;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f485e9f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f485e9f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x29293848;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x29293848;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5268;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x48486868;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x48486868;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7e8d;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x2e38;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x3f;
	destPtr += sizeof( unsigned char );

	destPtr += 13;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52689f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52689f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x292e385e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x292e385e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e7e;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f485e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f485e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7e9f;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x29383f52;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x29383f52;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x6873;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x8d;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f5273b4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f5273b4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09415779583034607827e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29292e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e738d9f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f485268;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f485268;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7e9f;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09411712906380479388e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29292e48;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x52687e8d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52739f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52739f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09739372145646735001e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2929385e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x687e9fcc;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x383f525e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x383f525e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7e8d;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xb4;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09739372083547789005e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2929385e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e8d9f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x48527e9f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x48527e9f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.52880317654352781891e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x292e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5eb4cc;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f486873;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f486873;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8db4;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 2.52880317673556195594e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x292e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e689fb4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f5e73b4;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f5e73b4;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.52556730089640292119e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x292e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68cce6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x38485e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x38485e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8db4;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09414515528016514524e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29292e5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5eb4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xcc;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x48527e9f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x48527e9f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 3.05713551277462332964e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e2e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685ecce6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x383f685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x383f685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8db4;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.09739367145638935285e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2929385e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685ecc;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x485e739f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x485e739f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.89998822842462422948e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68ccf7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x383f5e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x383f5e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8db4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xcc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.52556730069520198752e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x292e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e68cc;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x4852739f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4852739f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.90781221552244435337e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685ee6f7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3838685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3838685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8db4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xcc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 2.52880322610620726658e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x292e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5ee6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x485e739f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x485e739f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.89998822842462422948e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68ccf7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5eb4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.05322351921803239242e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e2e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685ee6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x4852739f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4852739f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.90781221552244435337e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685ee6f7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x68b4;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.89995766595491322084e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e68e6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x4852739f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x4852739f;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.89998822842462422948e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68ccf7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5ecc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 3.05713545307319151827e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e2e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685ee6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52738d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52738d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.90781221552244435337e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685ee6f7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x68cc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.89998822793815665151e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e68e6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52738d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52738d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.89998822842462422948e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68ccf7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5ecc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 4.90781221549756080350e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5ee6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52688d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52688d;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.90781221552244435337e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685ee6f7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x68e6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 7.12575456395409889133e-38;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x38383f68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685ee6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.21845593269069097364e-38;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x383f5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68e6f7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e3f5e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e3f5e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4e6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

#ifdef powerc
	*( ( double * )destPtr ) = 7.12570989688399353671e-38;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x38383f5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e68e6;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f485e7e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f485e7e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 9.19315650417901419138e-38;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x383f485e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685ef7fc;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e3f685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e3f685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4f7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 4.79243324999253440030e-04;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3f3f685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685ee6f7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f485e7e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f485e7e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9f;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 4.77934342416719065358e-04;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3f3f5268;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5eccf7fc;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x292e3f5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x292e3f5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x68cc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 4.77932013754414142855e-04;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3f3f525e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e68ccf7;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x38485e73;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x38485e73;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8d;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x485e5e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x485e5e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe6fc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x29384868;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x29384868;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9fe6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 1;

#ifdef powerc
	*( ( double * )destPtr ) = 4.77338295577929539425e-04;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x3f3f4868;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5ee6e6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x383f5e73;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x383f5e73;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8d;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x525e7e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x525e7e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe6fc;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4f7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x3f525e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x3f525e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5ecc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xf7;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x383f6868;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x383f6868;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x7e;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x5e738dcc;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5e738dcc;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xf7fc;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e3f5e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e3f5e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xccfc;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x48485e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x48485e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x68cc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x383f5e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x383f5e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x7e8d;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x739fccf7;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x739fccf7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr += 8;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x29384868;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x29384868;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9fe6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 2;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x48525e5e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x48525e5e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5ecc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xe6;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x737e;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x7e9fcce6;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7e9fcce6;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e38525e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38525e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4f7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x525e735e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x525e735e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4e6;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e385e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x687e;
	destPtr += sizeof( unsigned short );

	destPtr += 22;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e48689f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e48689f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe6fc;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 3;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x525e735e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x525e735e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4b4;
	destPtr += sizeof( unsigned short );

	destPtr += 9;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e73;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x73;
	destPtr += sizeof( unsigned char );

	destPtr += 20;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2e3f527e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2e3f527e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4f7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x5e687e9f;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5e687e9f;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x9f;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.05317526427736425484e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29385e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e687373;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 18;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x29384868;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x29384868;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x9fe6;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 4;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x5e687e8d;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5e687e8d;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x8d;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.52880322610692640482e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x292e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e6868;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0x68;
	destPtr += sizeof( unsigned char );

	destPtr += 16;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x2938485e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x2938485e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x8dcc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 5;

#ifdef powerc
	*( ( unsigned long * )destPtr ) = 0x6873737e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x6873737e;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.05317526427418101927e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29385e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e5e;
	destPtr += sizeof( unsigned short );

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = 4.03300909957342646497e-110;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x29383f5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7eccf7ff;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 6;

#ifdef powerc
#else
#endif
	*( ( unsigned short * )destPtr ) = 0x7373;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x68;
	destPtr += sizeof( unsigned char );

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 4.90781221549929242940e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e38685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0x5e5e;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0x52;
	destPtr += sizeof( unsigned char );

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 4.87560883009145772602e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e383f5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7eb4f7fc;
	destPtr += sizeof( unsigned long );
#endif

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 7;

#ifdef powerc
	*( ( double * )destPtr ) = 8.89436594035206841094e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.03429873595935375954e-86;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54933105777057631336e+194;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x68685e52;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x483f382e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2e2e2e38;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x385e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x52524848;
	destPtr += sizeof( unsigned long );
#endif

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 3.03825078467172855082e-86;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x2e2e3848;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e7eb4f7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 8;

#ifdef powerc
	*( ( double * )destPtr ) = 2.28779401627672935727e+146;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 7.12561107030965503836e-38;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54933105777057047811e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.69559549817852477788e-04;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5e525248;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3f383838;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x38383f48;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5252483f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3f38382e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x2e2e2e38;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3f52687e;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xb4f7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 9;

#ifdef powerc
	*( ( double * )destPtr ) = 2.28779432254296237473e+146;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223022228083065669e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223009080822319554e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.69563523493076739705e-04;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x688dccf7;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x5e525248;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5248483f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3f38383f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x3f48525e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x688dccf7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 10;

#ifdef powerc
	*( ( double * )destPtr ) = 2.28781337618401654949e+146;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223022228083065669e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223022270408772834e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.08585630075594235073e+146;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5e525252;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e8db4;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xe6f7;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 11;

#ifdef powerc
	*( ( double * )destPtr ) = 6.08585644960403270985e+146;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223022228083065669e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223022270408772834e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.08585758734277186695e+146;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb4ccf7fc;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 12;

#ifdef powerc
	*( ( double * )destPtr ) = 1.36000875089414434162e+248;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223022228083065669e+194;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 5.54223022270408772834e+194;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0x5ecce6f7;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x7373737e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x7e7e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e5e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5ecce6f7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfcff;
	destPtr += sizeof( unsigned short );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 14;

#ifdef powerc
	*( ( double * )destPtr ) = -2.16410466962475752970e-243;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.79700648120798792617e+146;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 3.79700730359790290477e+146;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0x8d8d8d9f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x8d9f8d68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e68;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e5e685e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xcce6e6f7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfcff;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 16;

#ifdef powerc
	*( ( double * )destPtr ) = -2.30928597520874570808e-156;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = 6.08581834232204502692e+146;
	destPtr += sizeof( double );
	*( ( unsigned long * )destPtr ) = 0xe6f7fcfc;
	destPtr += sizeof( unsigned long );
#else
	*( ( unsigned long * )destPtr ) = 0x9f9fb49f;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xb4b4b4b4;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x5e685e5e;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0x685e68e6;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe6f7fcfc;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfcff;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 19;

#ifdef powerc
	*( ( double * )destPtr ) = -8.44466160253093954257e-55;
	destPtr += sizeof( double );
	*( ( double * )destPtr ) = -4.99691957563006063956e+187;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xb4b4b4cc;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xcccccce6;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xe6e6f7f7;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf7fcfcff;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned char * )destPtr ) = 0xff;
	destPtr += sizeof( unsigned char );

	destPtr = rowStart;
	rowStart += sRowBytes;

	destPtr += 22;

#ifdef powerc
	*( ( double * )destPtr ) = -9.28873239453140708717e+61;
	destPtr += sizeof( double );
#else
	*( ( unsigned long * )destPtr ) = 0xcccce6e6;
	destPtr += sizeof( unsigned long );
	*( ( unsigned long * )destPtr ) = 0xf7f7f7f7;
	destPtr += sizeof( unsigned long );
#endif
	*( ( unsigned short * )destPtr ) = 0xfcfc;
	destPtr += sizeof( unsigned short );
	*( ( unsigned char * )destPtr ) = 0xfc;
	destPtr += sizeof( unsigned char );

}
